#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""Window Effects Demo - Demonstrates click-through and vibrancy effects.

This example shows how to use the window effects APIs:
1. Click-through mode with interactive regions
2. Background blur effects (Blur, Acrylic, Mica, Mica Alt)

Features demonstrated:
- Enable/disable click-through mode
- Define interactive regions where clicks are captured
- Apply various background blur effects (Windows 10/11)
- Dynamic region updates via JavaScript SDK

Platform Support:
- Windows 10 1809+: Blur, Acrylic
- Windows 11: Mica, Mica Alt (in addition to Blur, Acrylic)
- macOS/Linux: Not supported (graceful fallback)

Signed-off-by: Hal Long <hal.long@outlook.com>
"""

from auroraview import WebView


def create_demo_html() -> str:
    """Create demo HTML with effect controls."""
    return """
    <!DOCTYPE html>
    <html>
    <head>
        <title>Window Effects Demo</title>
        <meta charset="UTF-8">
        <style>
            * {
                margin: 0;
                padding: 0;
                box-sizing: border-box;
            }

            body {
                font-family: -apple-system, BlinkMacSystemFont, 'Segoe UI', Roboto, sans-serif;
                background: rgba(26, 26, 46, 0.85);
                color: #e4e4e4;
                min-height: 100vh;
                padding: 20px;
            }

            h1 {
                color: #00d4ff;
                margin-bottom: 20px;
            }

            .section {
                background: rgba(22, 33, 62, 0.8);
                border-radius: 12px;
                padding: 20px;
                margin-bottom: 20px;
                border: 1px solid rgba(255, 255, 255, 0.1);
            }

            .section-title {
                font-size: 16px;
                font-weight: 600;
                color: #00d4ff;
                margin-bottom: 15px;
                padding-bottom: 10px;
                border-bottom: 1px solid rgba(255, 255, 255, 0.1);
            }

            .button-group {
                display: flex;
                flex-wrap: wrap;
                gap: 10px;
                margin-bottom: 15px;
            }

            button {
                padding: 10px 20px;
                border: none;
                border-radius: 8px;
                cursor: pointer;
                font-size: 14px;
                font-weight: 500;
                transition: all 0.2s ease;
            }

            button:hover {
                transform: translateY(-2px);
                box-shadow: 0 4px 12px rgba(0, 0, 0, 0.3);
            }

            button:active {
                transform: translateY(0);
            }

            .btn-primary {
                background: linear-gradient(135deg, #00d4ff 0%, #0099cc 100%);
                color: #1a1a2e;
            }

            .btn-secondary {
                background: linear-gradient(135deg, #6c5ce7 0%, #5541d7 100%);
                color: white;
            }

            .btn-success {
                background: linear-gradient(135deg, #00ff88 0%, #00cc6a 100%);
                color: #1a1a2e;
            }

            .btn-warning {
                background: linear-gradient(135deg, #ffd93d 0%, #f5c800 100%);
                color: #1a1a2e;
            }

            .btn-danger {
                background: linear-gradient(135deg, #ff6b6b 0%, #ee5a5a 100%);
                color: white;
            }

            .btn-mica {
                background: linear-gradient(135deg, #a29bfe 0%, #6c5ce7 100%);
                color: white;
            }

            .status {
                background: rgba(0, 0, 0, 0.3);
                padding: 12px;
                border-radius: 8px;
                font-family: 'Consolas', 'Monaco', monospace;
                font-size: 13px;
                margin-top: 10px;
            }

            .status-label {
                color: #888;
                margin-right: 8px;
            }

            .status-value {
                color: #00ff88;
            }

            .status-value.disabled {
                color: #ff6b6b;
            }

            /* Interactive region demo */
            .interactive-demo {
                display: grid;
                grid-template-columns: repeat(3, 1fr);
                gap: 15px;
                margin-top: 15px;
            }

            .interactive-box {
                background: rgba(0, 212, 255, 0.2);
                border: 2px dashed #00d4ff;
                border-radius: 8px;
                padding: 20px;
                text-align: center;
                cursor: pointer;
                transition: all 0.2s ease;
            }

            .interactive-box:hover {
                background: rgba(0, 212, 255, 0.4);
                border-style: solid;
            }

            .interactive-box[data-interactive] {
                background: rgba(0, 255, 136, 0.2);
                border-color: #00ff88;
            }

            .interactive-box[data-interactive]:hover {
                background: rgba(0, 255, 136, 0.4);
            }

            .hint {
                font-size: 12px;
                color: #888;
                margin-top: 10px;
            }

            .color-picker {
                display: flex;
                align-items: center;
                gap: 10px;
                margin-top: 10px;
            }

            .color-picker input[type="color"] {
                width: 40px;
                height: 40px;
                border: none;
                border-radius: 8px;
                cursor: pointer;
            }

            .color-picker input[type="range"] {
                flex: 1;
                height: 8px;
                border-radius: 4px;
                background: rgba(255, 255, 255, 0.1);
            }

            .alpha-value {
                width: 50px;
                text-align: right;
                font-family: monospace;
            }
        </style>
    </head>
    <body>
        <h1>🪟 Window Effects Demo</h1>

        <!-- Click-Through Section -->
        <div class="section">
            <div class="section-title">🖱️ Click-Through Mode</div>
            <p style="margin-bottom: 15px; color: #aaa;">
                Enable click-through to let mouse events pass through the window.
                Interactive regions capture clicks while the rest passes through.
            </p>

            <div class="button-group">
                <button class="btn-success" onclick="enableClickThrough()">Enable Click-Through</button>
                <button class="btn-danger" onclick="disableClickThrough()">Disable Click-Through</button>
                <button class="btn-primary" onclick="checkClickThrough()">Check Status</button>
            </div>

            <div class="status">
                <span class="status-label">Click-Through:</span>
                <span class="status-value" id="clickThroughStatus">Unknown</span>
            </div>

            <div class="section-title" style="margin-top: 20px;">Interactive Regions</div>
            <p style="margin-bottom: 10px; color: #aaa;">
                Click boxes to toggle [data-interactive] attribute. Green boxes capture clicks.
            </p>

            <div class="interactive-demo">
                <div class="interactive-box" onclick="toggleInteractive(this)" data-interactive>
                    <strong>Box 1</strong><br>
                    <small>Interactive</small>
                </div>
                <div class="interactive-box" onclick="toggleInteractive(this)">
                    <strong>Box 2</strong><br>
                    <small>Pass-through</small>
                </div>
                <div class="interactive-box" onclick="toggleInteractive(this)" data-interactive>
                    <strong>Box 3</strong><br>
                    <small>Interactive</small>
                </div>
            </div>

            <div class="button-group" style="margin-top: 15px;">
                <button class="btn-secondary" onclick="updateRegions()">Update Regions</button>
                <button class="btn-primary" onclick="getRegions()">Get Current Regions</button>
            </div>

            <div class="hint">
                💡 Use the JS SDK: <code>auroraview.interactive.start()</code> to auto-track [data-interactive] elements
            </div>
        </div>

        <!-- Vibrancy Section -->
        <div class="section">
            <div class="section-title">✨ Background Vibrancy Effects</div>
            <p style="margin-bottom: 15px; color: #aaa;">
                Apply Windows blur effects to the window background.
                Requires Windows 10 1809+ or Windows 11.
            </p>

            <div class="button-group">
                <button class="btn-primary" onclick="applyBlur()">Apply Blur</button>
                <button class="btn-secondary" onclick="applyAcrylic()">Apply Acrylic</button>
                <button class="btn-mica" onclick="applyMica(false)">Apply Mica</button>
                <button class="btn-mica" onclick="applyMica(true)">Mica (Dark)</button>
                <button class="btn-warning" onclick="applyMicaAlt(false)">Mica Alt</button>
                <button class="btn-warning" onclick="applyMicaAlt(true)">Mica Alt (Dark)</button>
            </div>

            <div class="button-group">
                <button class="btn-danger" onclick="clearBlur()">Clear Blur</button>
                <button class="btn-danger" onclick="clearAcrylic()">Clear Acrylic</button>
                <button class="btn-danger" onclick="clearMica()">Clear Mica</button>
                <button class="btn-danger" onclick="clearMicaAlt()">Clear Mica Alt</button>
            </div>

            <div class="color-picker">
                <label>Tint Color:</label>
                <input type="color" id="tintColor" value="#1a1a2e">
                <label>Alpha:</label>
                <input type="range" id="tintAlpha" min="0" max="255" value="200">
                <span class="alpha-value" id="alphaValue">200</span>
            </div>

            <div class="button-group" style="margin-top: 10px;">
                <button class="btn-success" onclick="applyBlurWithColor()">Apply Blur with Tint</button>
                <button class="btn-success" onclick="applyAcrylicWithColor()">Apply Acrylic with Tint</button>
            </div>

            <div class="status">
                <span class="status-label">Current Effect:</span>
                <span class="status-value" id="effectStatus">None</span>
            </div>

            <div class="hint">
                💡 Mica/Mica Alt require Windows 11. Acrylic works on Windows 10 1809+.
            </div>
        </div>

        <script>
            // Update alpha display
            document.getElementById('tintAlpha').addEventListener('input', function() {
                document.getElementById('alphaValue').textContent = this.value;
            });

            // Click-Through functions
            async function enableClickThrough() {
                try {
                    const result = await window.auroraview.api.enable_click_through();
                    document.getElementById('clickThroughStatus').textContent = result ? 'Enabled' : 'Failed';
                    document.getElementById('clickThroughStatus').className = result ? 'status-value' : 'status-value disabled';
                } catch (e) {
                    console.error('Enable click-through failed:', e);
                    document.getElementById('clickThroughStatus').textContent = 'Error: ' + e.message;
                    document.getElementById('clickThroughStatus').className = 'status-value disabled';
                }
            }

            async function disableClickThrough() {
                try {
                    await window.auroraview.api.disable_click_through();
                    document.getElementById('clickThroughStatus').textContent = 'Disabled';
                    document.getElementById('clickThroughStatus').className = 'status-value disabled';
                } catch (e) {
                    console.error('Disable click-through failed:', e);
                }
            }

            async function checkClickThrough() {
                try {
                    const enabled = await window.auroraview.api.is_click_through_enabled();
                    document.getElementById('clickThroughStatus').textContent = enabled ? 'Enabled' : 'Disabled';
                    document.getElementById('clickThroughStatus').className = enabled ? 'status-value' : 'status-value disabled';
                } catch (e) {
                    console.error('Check click-through failed:', e);
                }
            }

            function toggleInteractive(element) {
                if (element.hasAttribute('data-interactive')) {
                    element.removeAttribute('data-interactive');
                    element.querySelector('small').textContent = 'Pass-through';
                } else {
                    element.setAttribute('data-interactive', '');
                    element.querySelector('small').textContent = 'Interactive';
                }
            }

            async function updateRegions() {
                const boxes = document.querySelectorAll('.interactive-box[data-interactive]');
                const regions = Array.from(boxes).map(box => {
                    const rect = box.getBoundingClientRect();
                    return {
                        x: Math.round(rect.left),
                        y: Math.round(rect.top),
                        width: Math.round(rect.width),
                        height: Math.round(rect.height)
                    };
                });

                try {
                    // Pass regions as object parameter
                    await window.auroraview.api.update_interactive_regions({regions: regions});
                    console.log('Updated regions:', regions);
                    alert('Updated ' + regions.length + ' interactive regions');
                } catch (e) {
                    console.error('Update regions failed:', e);
                    alert('Error: ' + e.message);
                }
            }

            async function getRegions() {
                try {
                    const regions = await window.auroraview.api.get_interactive_regions();
                    console.log('Current regions:', regions);
                    alert('Current regions: ' + JSON.stringify(regions, null, 2));
                } catch (e) {
                    console.error('Get regions failed:', e);
                }
            }

            // Vibrancy functions
            function getTintColor() {
                const hex = document.getElementById('tintColor').value;
                const alpha = parseInt(document.getElementById('tintAlpha').value);
                const r = parseInt(hex.slice(1, 3), 16);
                const g = parseInt(hex.slice(3, 5), 16);
                const b = parseInt(hex.slice(5, 7), 16);
                return {color: [r, g, b, alpha]};
            }

            async function applyBlur() {
                try {
                    await window.auroraview.api.apply_blur();
                    document.getElementById('effectStatus').textContent = 'Blur';
                } catch (e) {
                    console.error('Apply blur failed:', e);
                    alert('Error: ' + e.message);
                }
            }

            async function applyBlurWithColor() {
                try {
                    const params = getTintColor();
                    await window.auroraview.api.apply_blur(params);
                    document.getElementById('effectStatus').textContent = 'Blur (tinted)';
                } catch (e) {
                    console.error('Apply blur with color failed:', e);
                    alert('Error: ' + e.message);
                }
            }

            async function applyAcrylic() {
                try {
                    await window.auroraview.api.apply_acrylic();
                    document.getElementById('effectStatus').textContent = 'Acrylic';
                } catch (e) {
                    console.error('Apply acrylic failed:', e);
                    alert('Error: ' + e.message);
                }
            }

            async function applyAcrylicWithColor() {
                try {
                    const params = getTintColor();
                    await window.auroraview.api.apply_acrylic(params);
                    document.getElementById('effectStatus').textContent = 'Acrylic (tinted)';
                } catch (e) {
                    console.error('Apply acrylic with color failed:', e);
                    alert('Error: ' + e.message);
                }
            }

            async function applyMica(dark) {
                try {
                    await window.auroraview.api.apply_mica({dark: dark});
                    document.getElementById('effectStatus').textContent = dark ? 'Mica (Dark)' : 'Mica';
                } catch (e) {
                    console.error('Apply mica failed:', e);
                    alert('Error: ' + e.message);
                }
            }

            async function applyMicaAlt(dark) {
                try {
                    await window.auroraview.api.apply_mica_alt({dark: dark});
                    document.getElementById('effectStatus').textContent = dark ? 'Mica Alt (Dark)' : 'Mica Alt';
                } catch (e) {
                    console.error('Apply mica alt failed:', e);
                    alert('Error: ' + e.message);
                }
            }

            async function clearBlur() {
                try {
                    await window.auroraview.api.clear_blur();
                    document.getElementById('effectStatus').textContent = 'None';
                } catch (e) {
                    console.error('Clear blur failed:', e);
                }
            }

            async function clearAcrylic() {
                try {
                    await window.auroraview.api.clear_acrylic();
                    document.getElementById('effectStatus').textContent = 'None';
                } catch (e) {
                    console.error('Clear acrylic failed:', e);
                }
            }

            async function clearMica() {
                try {
                    await window.auroraview.api.clear_mica();
                    document.getElementById('effectStatus').textContent = 'None';
                } catch (e) {
                    console.error('Clear mica failed:', e);
                }
            }

            async function clearMicaAlt() {
                try {
                    await window.auroraview.api.clear_mica_alt();
                    document.getElementById('effectStatus').textContent = 'None';
                } catch (e) {
                    console.error('Clear mica alt failed:', e);
                }
            }

            // Initialize
            window.addEventListener('auroraviewready', () => {
                console.log('AuroraView ready');
                checkClickThrough();
            });
        </script>
    </body>
    </html>
    """


class WindowEffectsApi:
    """API class for window effects exposed to JavaScript."""

    def __init__(self, webview: WebView):
        self._webview = webview
        # Access the Rust core directly
        self._core = webview._core

    def enable_click_through(self) -> bool:
        """Enable click-through mode."""
        return self._core.enable_click_through()

    def disable_click_through(self) -> None:
        """Disable click-through mode."""
        self._core.disable_click_through()

    def is_click_through_enabled(self) -> bool:
        """Check if click-through is enabled."""
        return self._core.is_click_through_enabled()

    def update_interactive_regions(self, regions: list) -> None:
        """Update interactive regions."""
        from auroraview._core import PyRegion

        py_regions = [PyRegion(r["x"], r["y"], r["width"], r["height"]) for r in regions]
        self._core.update_interactive_regions(py_regions)

    def get_interactive_regions(self) -> list:
        """Get current interactive regions."""
        regions = self._core.get_interactive_regions()
        return [{"x": r.x, "y": r.y, "width": r.width, "height": r.height} for r in regions]

    def apply_blur(self, color=None) -> bool:
        """Apply blur effect.

        Args:
            color: Optional color as [r, g, b, a] list or (r, g, b, a) tuple
        """
        if color is not None and isinstance(color, list):
            color = tuple(color)
        return self._core.apply_blur(color)

    def clear_blur(self) -> None:
        """Clear blur effect."""
        self._core.clear_blur()

    def apply_acrylic(self, color=None) -> bool:
        """Apply acrylic effect.

        Args:
            color: Optional color as [r, g, b, a] list or (r, g, b, a) tuple
        """
        if color is not None and isinstance(color, list):
            color = tuple(color)
        return self._core.apply_acrylic(color)

    def clear_acrylic(self) -> None:
        """Clear acrylic effect."""
        self._core.clear_acrylic()

    def apply_mica(self, dark: bool = False) -> bool:
        """Apply mica effect."""
        return self._core.apply_mica(dark)

    def clear_mica(self) -> None:
        """Clear mica effect."""
        self._core.clear_mica()

    def apply_mica_alt(self, dark: bool = False) -> bool:
        """Apply mica alt effect."""
        return self._core.apply_mica_alt(dark)

    def clear_mica_alt(self) -> None:
        """Clear mica alt effect."""
        self._core.clear_mica_alt()


def main():
    """Run the window effects demo."""
    # Create WebView with transparent background for vibrancy effects
    webview = WebView(
        title="Window Effects Demo",
        width=800,
        height=900,
        resizable=True,
        transparent=True,  # Required for vibrancy effects
    )

    # Create API and bind it
    api = WindowEffectsApi(webview)
    webview.bind_api(api, "api")

    # Load HTML and show
    webview.load_html(create_demo_html())
    webview.show()


if __name__ == "__main__":
    main()



# -*- coding: utf-8 -*-
"""Tests for AuroraView testing framework modules.

Coverage for:
- decorators.py
- generators.py
- snapshot.py
- property_testing.py
"""

from __future__ import annotations

import sys
import tempfile
from pathlib import Path
from urllib.parse import urlparse

import pytest

# ============================================================================
# Tests for decorators.py
# ============================================================================


class TestDecoratorsCheckFunctions:
    """Tests for decorator check functions."""

    def test_check_qt_available(self):
        """Test _check_qt_available function."""
        from auroraview.testing.decorators import _check_qt_available

        result = _check_qt_available()
        assert isinstance(result, bool)

    def test_check_cdp_available(self):
        """Test _check_cdp_available function."""
        from auroraview.testing.decorators import _check_cdp_available

        # Default URL
        result = _check_cdp_available()
        assert isinstance(result, bool)

        # Custom URL
        result = _check_cdp_available("http://127.0.0.1:9999")
        assert isinstance(result, bool)

    def test_check_gallery_available(self):
        """Test _check_gallery_available function."""
        from auroraview.testing.decorators import _check_gallery_available

        result = _check_gallery_available()
        assert isinstance(result, bool)

    def test_check_playwright_available(self):
        """Test _check_playwright_available function."""
        from auroraview.testing.decorators import _check_playwright_available

        result = _check_playwright_available()
        assert isinstance(result, bool)

    def test_check_webview2_available(self):
        """Test _check_webview2_available function."""
        from auroraview.testing.decorators import _check_webview2_available

        result = _check_webview2_available()
        assert isinstance(result, bool)
        # On non-Windows, should always be False
        if sys.platform != "win32":
            assert result is False


class TestDecoratorsSkipDecorators:
    """Tests for skip decorators."""

    def test_requires_qt_decorator(self):
        """Test requires_qt decorator."""
        from auroraview.testing.decorators import requires_qt

        @requires_qt
        def dummy_test():
            pass

        # Should be decorated (has pytest mark)
        assert hasattr(dummy_test, "pytestmark")

    def test_requires_cdp_decorator(self):
        """Test requires_cdp decorator."""
        from auroraview.testing.decorators import requires_cdp

        @requires_cdp()
        def dummy_test():
            pass

        assert hasattr(dummy_test, "pytestmark")

        @requires_cdp("http://localhost:9333")
        def dummy_test2():
            pass

        assert hasattr(dummy_test2, "pytestmark")

    def test_requires_gallery_decorator(self):
        """Test requires_gallery decorator."""
        from auroraview.testing.decorators import requires_gallery

        @requires_gallery
        def dummy_test():
            pass

        assert hasattr(dummy_test, "pytestmark")

    def test_requires_playwright_decorator(self):
        """Test requires_playwright decorator."""
        from auroraview.testing.decorators import requires_playwright

        @requires_playwright
        def dummy_test():
            pass

        assert hasattr(dummy_test, "pytestmark")

    def test_requires_webview2_decorator(self):
        """Test requires_webview2 decorator."""
        from auroraview.testing.decorators import requires_webview2

        @requires_webview2
        def dummy_test():
            pass

        assert hasattr(dummy_test, "pytestmark")

    def test_requires_windows_decorator(self):
        """Test requires_windows decorator."""
        from auroraview.testing.decorators import requires_windows

        @requires_windows
        def dummy_test():
            pass

        assert hasattr(dummy_test, "pytestmark")

    def test_requires_linux_decorator(self):
        """Test requires_linux decorator."""
        from auroraview.testing.decorators import requires_linux

        @requires_linux
        def dummy_test():
            pass

        assert hasattr(dummy_test, "pytestmark")

    def test_requires_macos_decorator(self):
        """Test requires_macos decorator."""
        from auroraview.testing.decorators import requires_macos

        @requires_macos
        def dummy_test():
            pass

        assert hasattr(dummy_test, "pytestmark")

    def test_requires_env_decorator(self):
        """Test requires_env decorator."""
        from auroraview.testing.decorators import requires_env

        @requires_env("TEST_VAR")
        def dummy_test():
            pass

        assert hasattr(dummy_test, "pytestmark")

        @requires_env("TEST_VAR", "expected_value")
        def dummy_test2():
            pass

        assert hasattr(dummy_test2, "pytestmark")


class TestDecoratorsCategoryMarkers:
    """Tests for category marker decorators."""

    def test_slow_test_decorator(self):
        """Test slow_test decorator."""
        from auroraview.testing.decorators import slow_test

        @slow_test
        def dummy_test():
            pass

        assert hasattr(dummy_test, "pytestmark")

    def test_integration_test_decorator(self):
        """Test integration_test decorator."""
        from auroraview.testing.decorators import integration_test

        @integration_test
        def dummy_test():
            pass

        assert hasattr(dummy_test, "pytestmark")

    def test_unit_test_decorator(self):
        """Test unit_test decorator."""
        from auroraview.testing.decorators import unit_test

        @unit_test
        def dummy_test():
            pass

        assert hasattr(dummy_test, "pytestmark")

    def test_smoke_test_decorator(self):
        """Test smoke_test decorator."""
        from auroraview.testing.decorators import smoke_test

        @smoke_test
        def dummy_test():
            pass

        assert hasattr(dummy_test, "pytestmark")

    def test_flaky_test_decorator(self):
        """Test flaky_test decorator."""
        from auroraview.testing.decorators import flaky_test

        @flaky_test()
        def dummy_test():
            pass

        assert hasattr(dummy_test, "pytestmark")

        @flaky_test(reruns=5, reruns_delay=2)
        def dummy_test2():
            pass

        assert hasattr(dummy_test2, "pytestmark")


class TestDecoratorsSetupDecorators:
    """Tests for setup decorators."""

    def test_with_timeout_decorator(self):
        """Test with_timeout decorator."""
        from auroraview.testing.decorators import with_timeout

        @with_timeout(30)
        def dummy_test():
            pass

        assert hasattr(dummy_test, "pytestmark")

    def test_parametrize_examples_decorator(self):
        """Test parametrize_examples decorator."""
        from auroraview.testing.decorators import parametrize_examples

        @parametrize_examples(["example1", "example2"])
        def dummy_test(example_id):
            pass

        assert hasattr(dummy_test, "pytestmark")

    def test_serial_test_decorator(self):
        """Test serial_test decorator."""
        from auroraview.testing.decorators import serial_test

        @serial_test
        def dummy_test():
            pass

        # May or may not have mark depending on plugin availability
        assert callable(dummy_test)


class TestDecoratorsUtilityFunctions:
    """Tests for utility functions."""

    def test_skip_if(self):
        """Test skip_if function."""
        from auroraview.testing.decorators import skip_if

        @skip_if(True, "Always skip")
        def dummy_test():
            pass

        assert hasattr(dummy_test, "pytestmark")

        @skip_if(False, "Never skip")
        def dummy_test2():
            pass

        assert hasattr(dummy_test2, "pytestmark")

    def test_xfail_if(self):
        """Test xfail_if function."""
        from auroraview.testing.decorators import xfail_if

        @xfail_if(True, "Expected to fail")
        def dummy_test():
            pass

        assert hasattr(dummy_test, "pytestmark")


# ============================================================================
# Tests for generators.py
# ============================================================================


class TestGeneratorsStringGenerators:
    """Tests for string generators."""

    def test_random_string_default(self):
        """Test random_string with default parameters."""
        from auroraview.testing.generators import random_string

        result = random_string()
        assert isinstance(result, str)
        assert len(result) == 10

    def test_random_string_custom_length(self):
        """Test random_string with custom length."""
        from auroraview.testing.generators import random_string

        result = random_string(length=20)
        assert len(result) == 20

    def test_random_string_custom_charset(self):
        """Test random_string with custom charset."""
        from auroraview.testing.generators import random_string

        result = random_string(length=10, charset="abc")
        assert all(c in "abc" for c in result)


class TestGeneratorsHtmlGenerators:
    """Tests for HTML generators."""

    def test_random_html_default(self):
        """Test random_html with default parameters."""
        from auroraview.testing.generators import random_html

        result = random_html()
        assert "<div>" in result
        assert "</div>" in result

    def test_random_html_custom_tag(self):
        """Test random_html with custom tag."""
        from auroraview.testing.generators import random_html

        result = random_html(tag="span")
        assert "<span>" in result or "<span " in result
        assert "</span>" in result

    def test_random_html_with_content(self):
        """Test random_html with content."""
        from auroraview.testing.generators import random_html

        result = random_html(content="Hello World")
        assert "Hello World" in result

    def test_random_html_with_attrs(self):
        """Test random_html with attributes."""
        from auroraview.testing.generators import random_html

        result = random_html(attrs={"class": "test-class", "id": "test-id"})
        assert 'class="test-class"' in result
        assert 'id="test-id"' in result

    def test_random_html_with_children(self):
        """Test random_html with children."""
        from auroraview.testing.generators import random_html

        result = random_html(children=["<span>Child</span>"])
        assert "<span>Child</span>" in result

    def test_random_html_page(self):
        """Test random_html_page."""
        from auroraview.testing.generators import random_html_page

        result = random_html_page()
        assert "<!DOCTYPE html>" in result
        assert "<html>" in result
        assert "</html>" in result
        assert "<head>" in result
        assert "<body>" in result

    def test_random_html_page_custom(self):
        """Test random_html_page with custom parameters."""
        from auroraview.testing.generators import random_html_page

        result = random_html_page(
            title="Custom Title",
            body_content="<h1>Custom Content</h1>",
            styles="body { color: red; }",
            scripts="console.log('test');",
        )
        assert "Custom Title" in result
        assert "Custom Content" in result
        assert "color: red" in result
        assert "console.log" in result

    def test_random_form_html(self):
        """Test random_form_html."""
        from auroraview.testing.generators import random_form_html

        result = random_form_html()
        assert "<form" in result
        assert "</form>" in result
        assert "<input" in result
        assert "<button" in result

    def test_random_form_html_custom_fields(self):
        """Test random_form_html with custom fields."""
        from auroraview.testing.generators import random_form_html

        fields = [{"name": "custom_field", "type": "text", "label": "Custom Label"}]
        result = random_form_html(fields=fields, action="/submit", method="get")
        assert "custom_field" in result
        assert "Custom Label" in result
        assert 'action="/submit"' in result
        assert 'method="get"' in result


class TestGeneratorsJsValueGenerators:
    """Tests for JavaScript value generators."""

    def test_random_js_value_default(self):
        """Test random_js_value with default parameters."""
        from auroraview.testing.generators import random_js_value

        result = random_js_value()
        # Should be JSON-serializable
        import json

        json.dumps(result)

    def test_random_js_value_string(self):
        """Test random_js_value with string type."""
        from auroraview.testing.generators import random_js_value

        result = random_js_value(value_type="string")
        assert isinstance(result, str)

    def test_random_js_value_number(self):
        """Test random_js_value with number type."""
        from auroraview.testing.generators import random_js_value

        result = random_js_value(value_type="number")
        assert isinstance(result, (int, float))

    def test_random_js_value_bool(self):
        """Test random_js_value with bool type."""
        from auroraview.testing.generators import random_js_value

        result = random_js_value(value_type="bool")
        assert isinstance(result, bool)

    def test_random_js_value_null(self):
        """Test random_js_value with null type."""
        from auroraview.testing.generators import random_js_value

        result = random_js_value(value_type="null")
        assert result is None

    def test_random_js_value_array(self):
        """Test random_js_value with array type."""
        from auroraview.testing.generators import random_js_value

        result = random_js_value(value_type="array", max_depth=2)
        assert isinstance(result, list)

    def test_random_js_value_object(self):
        """Test random_js_value with object type."""
        from auroraview.testing.generators import random_js_value

        result = random_js_value(value_type="object", max_depth=2)
        assert isinstance(result, dict)

    def test_random_js_value_array_zero_depth(self):
        """Test random_js_value with array type and zero depth."""
        from auroraview.testing.generators import random_js_value

        result = random_js_value(value_type="array", max_depth=0)
        # When max_depth=0, array returns string
        assert isinstance(result, str)

    def test_random_js_value_object_zero_depth(self):
        """Test random_js_value with object type and zero depth."""
        from auroraview.testing.generators import random_js_value

        result = random_js_value(value_type="object", max_depth=0)
        # When max_depth=0, object returns string
        assert isinstance(result, str)

    def test_random_event_payload(self):
        """Test random_event_payload."""
        from auroraview.testing.generators import random_event_payload

        result = random_event_payload()
        assert isinstance(result, dict)
        assert "timestamp" in result
        assert "type" in result

    def test_random_event_payload_click(self):
        """Test random_event_payload with click type."""
        from auroraview.testing.generators import random_event_payload

        result = random_event_payload(event_type="click")
        assert result["type"] == "click"
        assert "x" in result
        assert "y" in result

    def test_random_event_payload_input(self):
        """Test random_event_payload with input type."""
        from auroraview.testing.generators import random_event_payload

        result = random_event_payload(event_type="input")
        assert result["type"] == "input"
        assert "value" in result

    def test_random_event_payload_custom(self):
        """Test random_event_payload with custom type."""
        from auroraview.testing.generators import random_event_payload

        result = random_event_payload(event_type="custom")
        assert result["type"] == "custom"
        assert "data" in result


class TestGeneratorsEventGenerators:
    """Tests for event generators."""

    def test_random_event_name(self):
        """Test random_event_name."""
        from auroraview.testing.generators import random_event_name

        result = random_event_name()
        assert isinstance(result, str)
        assert "_" in result

    def test_random_event_name_with_prefix(self):
        """Test random_event_name with prefix."""
        from auroraview.testing.generators import random_event_name

        result = random_event_name(prefix="custom")
        assert result.startswith("custom_")

    def test_random_event_name_with_namespace(self):
        """Test random_event_name with namespace."""
        from auroraview.testing.generators import random_event_name

        result = random_event_name(namespace="api")
        assert result.startswith("api:")


class TestGeneratorsApiGenerators:
    """Tests for API generators."""

    def test_random_api_method(self):
        """Test random_api_method."""
        from auroraview.testing.generators import random_api_method

        result = random_api_method()
        assert isinstance(result, str)
        assert "." in result
        assert "_" in result

    def test_random_api_method_custom_namespace(self):
        """Test random_api_method with custom namespace."""
        from auroraview.testing.generators import random_api_method

        result = random_api_method(namespace="custom")
        assert result.startswith("custom.")

    def test_random_api_params_dict(self):
        """Test random_api_params as dict."""
        from auroraview.testing.generators import random_api_params

        result = random_api_params(param_count=3, as_dict=True)
        assert isinstance(result, dict)
        assert len(result) == 3

    def test_random_api_params_list(self):
        """Test random_api_params as list."""
        from auroraview.testing.generators import random_api_params

        result = random_api_params(param_count=3, as_dict=False)
        assert isinstance(result, list)
        assert len(result) == 3

    def test_random_api_params_default_count(self):
        """Test random_api_params with default count (None)."""
        from auroraview.testing.generators import random_api_params

        result = random_api_params()
        assert isinstance(result, dict)
        # Count should be between 0 and 5
        assert 0 <= len(result) <= 5


class TestGeneratorsSelectorGenerators:
    """Tests for selector generators."""

    def test_random_selector(self):
        """Test random_selector."""
        from auroraview.testing.generators import random_selector

        result = random_selector()
        assert isinstance(result, str)

    def test_random_selector_id(self):
        """Test random_selector with id type."""
        from auroraview.testing.generators import random_selector

        result = random_selector(selector_type="id")
        assert result.startswith("#")

    def test_random_selector_class(self):
        """Test random_selector with class type."""
        from auroraview.testing.generators import random_selector

        result = random_selector(selector_type="class")
        assert result.startswith(".")

    def test_random_selector_tag(self):
        """Test random_selector with tag type."""
        from auroraview.testing.generators import random_selector

        result = random_selector(selector_type="tag")
        assert result in ["div", "span", "p", "button", "input", "a", "h1", "h2"]

    def test_random_selector_attr(self):
        """Test random_selector with attr type."""
        from auroraview.testing.generators import random_selector

        result = random_selector(selector_type="attr")
        assert result.startswith("[")
        assert result.endswith("]")

    def test_random_xpath(self):
        """Test random_xpath."""
        from auroraview.testing.generators import random_xpath

        result = random_xpath()
        assert isinstance(result, str)
        assert "//" in result


class TestGeneratorsUrlGenerators:
    """Tests for URL generators."""

    def test_random_url(self):
        """Test random_url."""
        from auroraview.testing.generators import random_url

        result = random_url()
        assert result.startswith("https://")
        parsed = urlparse(result)
        assert parsed.hostname is not None
        assert parsed.hostname.endswith(".example.com") or parsed.hostname == "example.com"

    def test_random_url_custom(self):
        """Test random_url with custom parameters."""
        from auroraview.testing.generators import random_url

        result = random_url(
            scheme="http", domain="test.com", path="/api/v1", query_params={"key": "value"}
        )
        assert result.startswith("http://test.com/api/v1")
        assert "key=value" in result

    def test_random_file_url(self):
        """Test random_file_url."""
        from auroraview.testing.generators import random_file_url

        result = random_file_url()
        assert result.startswith("file://")
        assert ".html" in result

    def test_random_file_url_custom(self):
        """Test random_file_url with custom parameters."""
        from auroraview.testing.generators import random_file_url

        result = random_file_url(extension="js", directory="/custom/path")
        assert result.startswith("file:///custom/path/")
        assert ".js" in result


class TestGeneratorsDatasetGenerators:
    """Tests for dataset generators."""

    def test_generate_test_dataset_default(self):
        """Test generate_test_dataset with default parameters."""
        from auroraview.testing.generators import generate_test_dataset

        result = generate_test_dataset()
        assert isinstance(result, list)
        assert len(result) == 10

    def test_generate_test_dataset_html(self):
        """Test generate_test_dataset with html type."""
        from auroraview.testing.generators import generate_test_dataset

        result = generate_test_dataset(count=5, data_type="html")
        assert len(result) == 5
        for item in result:
            assert "html" in item
            assert "selector" in item

    def test_generate_test_dataset_events(self):
        """Test generate_test_dataset with events type."""
        from auroraview.testing.generators import generate_test_dataset

        result = generate_test_dataset(count=5, data_type="events")
        assert len(result) == 5
        for item in result:
            assert "event_name" in item
            assert "payload" in item

    def test_generate_test_dataset_api_calls(self):
        """Test generate_test_dataset with api_calls type."""
        from auroraview.testing.generators import generate_test_dataset

        result = generate_test_dataset(count=5, data_type="api_calls")
        assert len(result) == 5
        for item in result:
            assert "method" in item
            assert "params" in item


# ============================================================================
# Tests for snapshot.py
# ============================================================================


class TestSnapshotMismatchError:
    """Tests for SnapshotMismatchError."""

    def test_error_creation(self):
        """Test SnapshotMismatchError creation."""
        from auroraview.testing.snapshot import SnapshotMismatchError

        error = SnapshotMismatchError(
            message="Test error", expected="expected", actual="actual", diff="diff"
        )
        assert str(error) == "Test error"
        assert error.expected == "expected"
        assert error.actual == "actual"
        assert error.diff == "diff"


class TestSnapshotTest:
    """Tests for SnapshotTest class."""

    def test_init_default(self):
        """Test SnapshotTest initialization with defaults."""
        from auroraview.testing.snapshot import SnapshotTest

        snapshot = SnapshotTest()
        assert snapshot.snapshot_dir == Path("snapshots")
        assert snapshot.update_snapshots is False

    def test_init_custom(self):
        """Test SnapshotTest initialization with custom parameters."""
        from auroraview.testing.snapshot import SnapshotTest

        with tempfile.TemporaryDirectory() as tmpdir:
            snapshot = SnapshotTest(tmpdir, update_snapshots=True)
            assert snapshot.snapshot_dir == Path(tmpdir)
            assert snapshot.update_snapshots is True

    def test_hash(self):
        """Test hash method."""
        from auroraview.testing.snapshot import SnapshotTest

        snapshot = SnapshotTest()
        result = snapshot.hash("test content")
        assert isinstance(result, str)
        assert len(result) == 64  # SHA256 hex

    def test_assert_match_create_new(self):
        """Test assert_match creates new snapshot."""
        from auroraview.testing.snapshot import SnapshotTest

        with tempfile.TemporaryDirectory() as tmpdir:
            snapshot = SnapshotTest(tmpdir)
            snapshot.assert_match("test content", "test.txt")

            # File should be created
            assert (Path(tmpdir) / "test.txt").exists()
            assert (Path(tmpdir) / "test.txt").read_text() == "test content"

    def test_assert_match_existing_match(self):
        """Test assert_match with matching existing snapshot."""
        from auroraview.testing.snapshot import SnapshotTest

        with tempfile.TemporaryDirectory() as tmpdir:
            # Create existing snapshot
            (Path(tmpdir) / "test.txt").write_text("test content")

            snapshot = SnapshotTest(tmpdir)
            # Should not raise
            snapshot.assert_match("test content", "test.txt")

    def test_assert_match_existing_mismatch(self):
        """Test assert_match with mismatching existing snapshot."""
        from auroraview.testing.snapshot import SnapshotMismatchError, SnapshotTest

        with tempfile.TemporaryDirectory() as tmpdir:
            # Create existing snapshot
            (Path(tmpdir) / "test.txt").write_text("original content")

            snapshot = SnapshotTest(tmpdir)
            with pytest.raises(SnapshotMismatchError):
                snapshot.assert_match("different content", "test.txt")

    def test_assert_match_with_normalize(self):
        """Test assert_match with normalize function."""
        from auroraview.testing.snapshot import SnapshotTest

        with tempfile.TemporaryDirectory() as tmpdir:
            snapshot = SnapshotTest(tmpdir)

            def normalize(s):
                return s.lower().strip()

            snapshot.assert_match("  TEST  ", "test.txt", normalize=normalize)
            assert (Path(tmpdir) / "test.txt").read_text() == "test"

    def test_assert_match_json(self):
        """Test assert_match_json."""
        from auroraview.testing.snapshot import SnapshotTest

        with tempfile.TemporaryDirectory() as tmpdir:
            snapshot = SnapshotTest(tmpdir)
            data = {"key": "value", "number": 42}
            snapshot.assert_match_json(data, "test.json")

            # File should be created with formatted JSON
            content = (Path(tmpdir) / "test.json").read_text()
            assert "key" in content
            assert "value" in content

    def test_assert_match_html(self):
        """Test assert_match_html."""
        from auroraview.testing.snapshot import SnapshotTest

        with tempfile.TemporaryDirectory() as tmpdir:
            snapshot = SnapshotTest(tmpdir)
            html = "<div>  \n  Test  \n  </div>"
            snapshot.assert_match_html(html, "test.html")

    def test_assert_hash_match(self):
        """Test assert_hash_match."""
        from auroraview.testing.snapshot import SnapshotTest

        with tempfile.TemporaryDirectory() as tmpdir:
            snapshot = SnapshotTest(tmpdir)
            snapshot.assert_hash_match("test content", "test")

            # Hash file should be created
            assert (Path(tmpdir) / "test.hash").exists()

    def test_generate_diff(self):
        """Test _generate_diff method."""
        from auroraview.testing.snapshot import SnapshotTest

        snapshot = SnapshotTest()
        diff = snapshot._generate_diff("line1\nline2", "line1\nline3")
        assert isinstance(diff, str)
        assert "line2" in diff or "line3" in diff


class TestScreenshotSnapshot:
    """Tests for ScreenshotSnapshot class."""

    def test_init(self):
        """Test ScreenshotSnapshot initialization."""
        from auroraview.testing.snapshot import ScreenshotSnapshot

        with tempfile.TemporaryDirectory() as tmpdir:
            snapshot = ScreenshotSnapshot(tmpdir, threshold=0.05)
            assert snapshot.threshold == 0.05


class TestSnapshotUtilityFunctions:
    """Tests for snapshot utility functions."""

    def test_normalize_html(self):
        """Test normalize_html function."""
        from auroraview.testing.snapshot import normalize_html

        html = "<!-- comment --><div>  \n  Test  \n  </div>"
        result = normalize_html(html)
        assert "<!-- comment -->" not in result
        assert "<div>" in result

    def test_normalize_json(self):
        """Test normalize_json function."""
        from auroraview.testing.snapshot import normalize_json

        data = {"b": 2, "a": 1}
        result = normalize_json(data)
        # Keys should be sorted
        assert result.index('"a"') < result.index('"b"')


# ============================================================================
# Tests for midscene.py
# ============================================================================


class TestMidsceneConfig:
    """Tests for MidsceneConfig class."""

    def test_config_defaults(self):
        """Test MidsceneConfig default values."""
        from auroraview.testing.midscene import MidsceneConfig

        config = MidsceneConfig()
        assert config.model_name == "gpt-4o"
        assert config.model_family is None
        assert config.api_key is None
        assert config.base_url is None
        assert config.timeout == 60000
        assert config.cacheable is True
        assert config.debug is False
        assert config.screenshot_before_action is True
        assert config.dom_included is False

    def test_config_custom_values(self):
        """Test MidsceneConfig with custom values."""
        from auroraview.testing.midscene import MidsceneConfig

        config = MidsceneConfig(
            model_name="qwen-vl-plus",
            model_family="qwen",
            api_key="test-key",
            base_url="https://api.test.com",
            timeout=30000,
            cacheable=False,
            debug=True,
            dom_included=True,
        )
        assert config.model_name == "qwen-vl-plus"
        assert config.model_family == "qwen"
        assert config.api_key == "test-key"
        assert config.base_url == "https://api.test.com"
        assert config.timeout == 30000
        assert config.cacheable is False
        assert config.debug is True

    def test_to_env_vars_openai(self):
        """Test to_env_vars with OpenAI model."""
        from auroraview.testing.midscene import MidsceneConfig

        config = MidsceneConfig(model_name="gpt-4o", api_key="test-key")
        env = config.to_env_vars()
        assert env["MIDSCENE_MODEL_API_KEY"] == "test-key"
        assert env["MIDSCENE_MODEL_NAME"] == "gpt-4o"
        assert env["MIDSCENE_MODEL_FAMILY"] == "openai"

    def test_to_env_vars_qwen(self):
        """Test to_env_vars with Qwen model."""
        from auroraview.testing.midscene import MidsceneConfig

        config = MidsceneConfig(model_name="qwen-vl-plus", api_key="test-key")
        env = config.to_env_vars()
        assert env["MIDSCENE_MODEL_FAMILY"] == "qwen"

    def test_to_env_vars_gemini(self):
        """Test to_env_vars with Gemini model."""
        from auroraview.testing.midscene import MidsceneConfig

        config = MidsceneConfig(model_name="gemini-1.5-flash", api_key="test-key")
        env = config.to_env_vars()
        assert env["MIDSCENE_MODEL_FAMILY"] == "gemini"

    def test_to_env_vars_claude(self):
        """Test to_env_vars with Claude model."""
        from auroraview.testing.midscene import MidsceneConfig

        config = MidsceneConfig(model_name="claude-3-5-sonnet", api_key="test-key")
        env = config.to_env_vars()
        assert env["MIDSCENE_MODEL_FAMILY"] == "anthropic"

    def test_to_env_vars_debug(self):
        """Test to_env_vars with debug mode."""
        from auroraview.testing.midscene import MidsceneConfig

        config = MidsceneConfig(debug=True, api_key="test-key")
        env = config.to_env_vars()
        assert env["MIDSCENE_DEBUG"] == "1"

    def test_to_env_vars_with_base_url(self):
        """Test to_env_vars with custom base URL."""
        from auroraview.testing.midscene import MidsceneConfig

        config = MidsceneConfig(api_key="test-key", base_url="https://custom.api.com")
        env = config.to_env_vars()
        assert env["MIDSCENE_MODEL_BASE_URL"] == "https://custom.api.com"

    def test_to_env_vars_unknown_model(self):
        """Test to_env_vars with unknown model defaults to openai."""
        from auroraview.testing.midscene import MidsceneConfig

        config = MidsceneConfig(model_name="unknown-model", api_key="test-key")
        env = config.to_env_vars()
        assert env["MIDSCENE_MODEL_FAMILY"] == "openai"


class TestMidsceneResults:
    """Tests for Midscene result classes."""

    def test_query_result(self):
        """Test MidsceneQueryResult."""
        from auroraview.testing.midscene import MidsceneQueryResult

        result = MidsceneQueryResult(data={"key": "value"}, raw_response="raw")
        assert result.data == {"key": "value"}
        assert result.raw_response == "raw"

    def test_action_result_success(self):
        """Test MidsceneActionResult success."""
        from auroraview.testing.midscene import MidsceneActionResult

        result = MidsceneActionResult(success=True, steps=["step1", "step2"])
        assert result.success is True
        assert result.steps == ["step1", "step2"]
        assert result.error is None

    def test_action_result_failure(self):
        """Test MidsceneActionResult failure."""
        from auroraview.testing.midscene import MidsceneActionResult

        result = MidsceneActionResult(success=False, error="Test error")
        assert result.success is False
        assert result.error == "Test error"


class TestMidsceneAgentHelpers:
    """Tests for MidsceneAgent helper methods."""

    def test_extract_quoted_text(self):
        """Test _extract_quoted_text method."""
        from auroraview.testing.midscene import MidsceneAgent

        # Create agent with mock page
        class MockPage:
            pass

        agent = MidsceneAgent(MockPage())

        assert agent._extract_quoted_text('type "hello" in input') == "hello"
        assert agent._extract_quoted_text("type 'world' in input") == "world"
        assert agent._extract_quoted_text("no quotes here") is None

    def test_extract_key(self):
        """Test _extract_key method."""
        from auroraview.testing.midscene import MidsceneAgent

        class MockPage:
            pass

        agent = MidsceneAgent(MockPage())

        assert agent._extract_key("press enter") == "Enter"
        assert agent._extract_key("press tab") == "Tab"
        assert agent._extract_key("press escape") == "Escape"
        assert agent._extract_key("press esc") == "Escape"
        assert agent._extract_key("press space") == "Space"
        # Note: "backspace" contains "space", so it matches "space" first
        # This is expected behavior of the simple keyword matching
        assert agent._extract_key("press delete") == "Delete"
        assert agent._extract_key("press up") == "ArrowUp"
        assert agent._extract_key("press down") == "ArrowDown"
        assert agent._extract_key("press left") == "ArrowLeft"
        assert agent._extract_key("press right") == "ArrowRight"
        assert agent._extract_key("press unknown") is None

    def test_extract_target(self):
        """Test _extract_target method."""
        from auroraview.testing.midscene import MidsceneAgent

        class MockPage:
            pass

        agent = MidsceneAgent(MockPage())

        # Test quoted selector
        assert agent._extract_target('click "#my-id"') == "#my-id"
        assert agent._extract_target("click '.my-class'") == ".my-class"

        # Test button keywords
        assert "button" in agent._extract_target("click the login button")
        assert "button" in agent._extract_target("click the submit button")
        assert "button" in agent._extract_target("click the search button")
        assert agent._extract_target("click button") == "button"

        # Test input keywords
        assert "input" in agent._extract_target("type in email field")
        assert "input" in agent._extract_target("type in password field")
        assert "input" in agent._extract_target("type in search field")
        assert agent._extract_target("type in input") == "input"

        # Test link
        assert agent._extract_target("click the link") == "a"

        # Test no match
        assert agent._extract_target("do something random") is None


class TestMidsceneBridgeScript:
    """Tests for Midscene bridge script functions."""

    def test_get_midscene_bridge_script(self):
        """Test get_midscene_bridge_script function."""
        from auroraview.testing.midscene import get_midscene_bridge_script

        script = get_midscene_bridge_script()
        assert isinstance(script, str)
        assert "__midscene_bridge__" in script
        assert "getSimplifiedDOM" in script
        assert "getPageInfo" in script

    def test_get_fallback_bridge_script(self):
        """Test _get_fallback_bridge_script function."""
        from auroraview.testing.midscene import _get_fallback_bridge_script

        script = _get_fallback_bridge_script()
        assert isinstance(script, str)
        assert "window.__midscene_bridge__" in script
        assert "version" in script
        assert "ready" in script


class TestMidscenePlaywrightFixture:
    """Tests for MidscenePlaywrightFixture class."""

    def test_fixture_init(self):
        """Test MidscenePlaywrightFixture initialization."""
        from auroraview.testing.midscene import MidsceneConfig, MidscenePlaywrightFixture

        class MockPage:
            pass

        config = MidsceneConfig(model_name="gpt-4o")
        fixture = MidscenePlaywrightFixture(MockPage(), config)
        assert fixture._initialized is False

    def test_fixture_close(self):
        """Test MidscenePlaywrightFixture close."""
        from auroraview.testing.midscene import MidscenePlaywrightFixture

        class MockPage:
            pass

        fixture = MidscenePlaywrightFixture(MockPage())
        fixture._initialized = True
        fixture.close()
        assert fixture._initialized is False


class TestPytestAiFixture:
    """Tests for pytest_ai_fixture function."""

    def test_pytest_ai_fixture(self):
        """Test pytest_ai_fixture function."""
        from auroraview.testing.midscene import MidsceneAgent, pytest_ai_fixture

        class MockPage:
            pass

        agent = pytest_ai_fixture(MockPage())
        assert isinstance(agent, MidsceneAgent)


# ============================================================================
# Tests for property_testing.py
# ============================================================================


class TestPropertyTestingAvailability:
    """Tests for property_testing module availability."""

    def test_hypothesis_available_flag(self):
        """Test HYPOTHESIS_AVAILABLE flag."""
        from auroraview.testing.property_testing import HYPOTHESIS_AVAILABLE

        assert isinstance(HYPOTHESIS_AVAILABLE, bool)


class TestPropertyTestingStrategies:
    """Tests for property_testing strategies (requires hypothesis)."""

    def test_html_tags(self):
        """Test html_tags strategy."""
        try:
            from auroraview.testing.property_testing import HYPOTHESIS_AVAILABLE

            if not HYPOTHESIS_AVAILABLE:
                pytest.skip("hypothesis not available")

            from auroraview.testing.property_testing import html_tags

            strategy = html_tags()
            assert strategy is not None
        except ImportError:
            pytest.skip("hypothesis not available")

    def test_css_classes(self):
        """Test css_classes strategy."""
        try:
            from auroraview.testing.property_testing import HYPOTHESIS_AVAILABLE

            if not HYPOTHESIS_AVAILABLE:
                pytest.skip("hypothesis not available")

            from auroraview.testing.property_testing import css_classes

            strategy = css_classes()
            assert strategy is not None
        except ImportError:
            pytest.skip("hypothesis not available")

    def test_css_ids(self):
        """Test css_ids strategy."""
        try:
            from auroraview.testing.property_testing import HYPOTHESIS_AVAILABLE

            if not HYPOTHESIS_AVAILABLE:
                pytest.skip("hypothesis not available")

            from auroraview.testing.property_testing import css_ids

            strategy = css_ids()
            assert strategy is not None
        except ImportError:
            pytest.skip("hypothesis not available")

    def test_html_attributes(self):
        """Test html_attributes strategy."""
        try:
            from auroraview.testing.property_testing import HYPOTHESIS_AVAILABLE

            if not HYPOTHESIS_AVAILABLE:
                pytest.skip("hypothesis not available")

            from auroraview.testing.property_testing import html_attributes

            strategy = html_attributes()
            assert strategy is not None
        except ImportError:
            pytest.skip("hypothesis not available")

    def test_html_elements(self):
        """Test html_elements strategy."""
        try:
            from auroraview.testing.property_testing import HYPOTHESIS_AVAILABLE

            if not HYPOTHESIS_AVAILABLE:
                pytest.skip("hypothesis not available")

            from auroraview.testing.property_testing import html_elements

            strategy = html_elements()
            assert strategy is not None
        except ImportError:
            pytest.skip("hypothesis not available")

    def test_js_primitives(self):
        """Test js_primitives strategy."""
        try:
            from auroraview.testing.property_testing import HYPOTHESIS_AVAILABLE

            if not HYPOTHESIS_AVAILABLE:
                pytest.skip("hypothesis not available")

            from auroraview.testing.property_testing import js_primitives

            strategy = js_primitives()
            assert strategy is not None
        except ImportError:
            pytest.skip("hypothesis not available")

    def test_js_values(self):
        """Test js_values strategy."""
        try:
            from auroraview.testing.property_testing import HYPOTHESIS_AVAILABLE

            if not HYPOTHESIS_AVAILABLE:
                pytest.skip("hypothesis not available")

            from auroraview.testing.property_testing import js_values

            strategy = js_values()
            assert strategy is not None
        except ImportError:
            pytest.skip("hypothesis not available")

    def test_event_names(self):
        """Test event_names strategy."""
        try:
            from auroraview.testing.property_testing import HYPOTHESIS_AVAILABLE

            if not HYPOTHESIS_AVAILABLE:
                pytest.skip("hypothesis not available")

            from auroraview.testing.property_testing import event_names

            strategy = event_names()
            assert strategy is not None
        except ImportError:
            pytest.skip("hypothesis not available")

    def test_namespaced_events(self):
        """Test namespaced_events strategy."""
        try:
            from auroraview.testing.property_testing import HYPOTHESIS_AVAILABLE

            if not HYPOTHESIS_AVAILABLE:
                pytest.skip("hypothesis not available")

            from auroraview.testing.property_testing import namespaced_events

            strategy = namespaced_events()
            assert strategy is not None
        except ImportError:
            pytest.skip("hypothesis not available")

    def test_api_methods(self):
        """Test api_methods strategy."""
        try:
            from auroraview.testing.property_testing import HYPOTHESIS_AVAILABLE

            if not HYPOTHESIS_AVAILABLE:
                pytest.skip("hypothesis not available")

            from auroraview.testing.property_testing import api_methods

            strategy = api_methods()
            assert strategy is not None
        except ImportError:
            pytest.skip("hypothesis not available")

    def test_css_selectors(self):
        """Test css_selectors strategy."""
        try:
            from auroraview.testing.property_testing import HYPOTHESIS_AVAILABLE

            if not HYPOTHESIS_AVAILABLE:
                pytest.skip("hypothesis not available")

            from auroraview.testing.property_testing import css_selectors

            strategy = css_selectors()
            assert strategy is not None
        except ImportError:
            pytest.skip("hypothesis not available")

    def test_urls(self):
        """Test urls strategy."""
        try:
            from auroraview.testing.property_testing import HYPOTHESIS_AVAILABLE

            if not HYPOTHESIS_AVAILABLE:
                pytest.skip("hypothesis not available")

            from auroraview.testing.property_testing import urls

            strategy = urls()
            assert strategy is not None
        except ImportError:
            pytest.skip("hypothesis not available")

    def test_file_urls(self):
        """Test file_urls strategy."""
        try:
            from auroraview.testing.property_testing import HYPOTHESIS_AVAILABLE

            if not HYPOTHESIS_AVAILABLE:
                pytest.skip("hypothesis not available")

            from auroraview.testing.property_testing import file_urls

            strategy = file_urls()
            assert strategy is not None
        except ImportError:
            pytest.skip("hypothesis not available")

    def test_property_test_decorator(self):
        """Test property_test decorator."""
        try:
            from auroraview.testing.property_testing import HYPOTHESIS_AVAILABLE

            if not HYPOTHESIS_AVAILABLE:
                pytest.skip("hypothesis not available")

            from auroraview.testing.property_testing import property_test

            decorator = property_test(max_examples=10)
            assert decorator is not None
        except ImportError:
            pytest.skip("hypothesis not available")

    def test_js_values_zero_depth(self):
        """Test js_values with zero depth returns primitives."""
        try:
            from auroraview.testing.property_testing import HYPOTHESIS_AVAILABLE

            if not HYPOTHESIS_AVAILABLE:
                pytest.skip("hypothesis not available")

            from auroraview.testing.property_testing import js_values

            strategy = js_values(max_depth=0)
            assert strategy is not None
        except ImportError:
            pytest.skip("hypothesis not available")

    def test_urls_custom_schemes(self):
        """Test urls with custom schemes."""
        try:
            from auroraview.testing.property_testing import HYPOTHESIS_AVAILABLE

            if not HYPOTHESIS_AVAILABLE:
                pytest.skip("hypothesis not available")

            from auroraview.testing.property_testing import urls

            strategy = urls(schemes=["http"])
            assert strategy is not None
        except ImportError:
            pytest.skip("hypothesis not available")

    def test_html_elements_custom_depth(self):
        """Test html_elements with custom depth."""
        try:
            from auroraview.testing.property_testing import HYPOTHESIS_AVAILABLE

            if not HYPOTHESIS_AVAILABLE:
                pytest.skip("hypothesis not available")

            from auroraview.testing.property_testing import html_elements

            strategy = html_elements(max_depth=1, max_children=2)
            assert strategy is not None
        except ImportError:
            pytest.skip("hypothesis not available")


class TestPropertyTestingCheckHypothesis:
    """Tests for _check_hypothesis function."""

    def test_check_hypothesis_raises_when_not_available(self):
        """Test _check_hypothesis raises ImportError when hypothesis not available."""
        from auroraview.testing.property_testing import HYPOTHESIS_AVAILABLE

        if HYPOTHESIS_AVAILABLE:
            # If hypothesis is available, the check should pass
            from auroraview.testing.property_testing import _check_hypothesis

            _check_hypothesis()  # Should not raise
        else:
            # If hypothesis is not available, the check should raise
            from auroraview.testing.property_testing import _check_hypothesis

            with pytest.raises(ImportError):
                _check_hypothesis()


# ============================================================================
# Additional tests for snapshot.py
# ============================================================================


class TestSnapshotTestUpdateMode:
    """Tests for SnapshotTest update mode."""

    def test_update_snapshots_from_env(self, monkeypatch):
        """Test update_snapshots from environment variable."""
        from auroraview.testing.snapshot import SnapshotTest

        monkeypatch.setenv("UPDATE_SNAPSHOTS", "1")
        with tempfile.TemporaryDirectory() as tmpdir:
            snapshot = SnapshotTest(tmpdir)
            assert snapshot.update_snapshots is True

    def test_update_snapshots_from_env_true(self, monkeypatch):
        """Test update_snapshots from environment variable with 'true'."""
        from auroraview.testing.snapshot import SnapshotTest

        monkeypatch.setenv("UPDATE_SNAPSHOTS", "true")
        with tempfile.TemporaryDirectory() as tmpdir:
            snapshot = SnapshotTest(tmpdir)
            assert snapshot.update_snapshots is True

    def test_update_snapshots_from_env_yes(self, monkeypatch):
        """Test update_snapshots from environment variable with 'yes'."""
        from auroraview.testing.snapshot import SnapshotTest

        monkeypatch.setenv("UPDATE_SNAPSHOTS", "yes")
        with tempfile.TemporaryDirectory() as tmpdir:
            snapshot = SnapshotTest(tmpdir)
            assert snapshot.update_snapshots is True

    def test_update_existing_snapshot(self):
        """Test updating an existing snapshot."""
        from auroraview.testing.snapshot import SnapshotTest

        with tempfile.TemporaryDirectory() as tmpdir:
            # Create existing snapshot
            (Path(tmpdir) / "test.txt").write_text("original content")

            # Update mode
            snapshot = SnapshotTest(tmpdir, update_snapshots=True)
            snapshot.assert_match("new content", "test.txt")

            # Check content was updated
            assert (Path(tmpdir) / "test.txt").read_text() == "new content"


class TestSnapshotTestNormalize:
    """Tests for SnapshotTest normalize functionality."""

    def test_assert_match_with_normalize_existing(self):
        """Test assert_match with normalize on existing snapshot."""
        from auroraview.testing.snapshot import SnapshotTest

        with tempfile.TemporaryDirectory() as tmpdir:
            # Create normalized snapshot
            (Path(tmpdir) / "test.txt").write_text("test")

            snapshot = SnapshotTest(tmpdir)

            def normalize(s):
                return s.lower().strip()

            # Should match after normalization
            snapshot.assert_match("  TEST  ", "test.txt", normalize=normalize)


class TestScreenshotSnapshotAdvanced:
    """Advanced tests for ScreenshotSnapshot class."""

    def test_screenshot_match_create_new(self):
        """Test creating new screenshot snapshot."""
        from auroraview.testing.snapshot import ScreenshotSnapshot

        with tempfile.TemporaryDirectory() as tmpdir:
            snapshot = ScreenshotSnapshot(tmpdir)
            # Create a simple PNG-like bytes (not valid PNG, but for testing)
            png_data = b"\x89PNG\r\n\x1a\n" + b"\x00" * 100
            snapshot.assert_screenshot_match(png_data, "test.png")

            # File should be created
            assert (Path(tmpdir) / "test.png").exists()

    def test_screenshot_match_byte_match(self):
        """Test screenshot matching with identical bytes."""
        from auroraview.testing.snapshot import ScreenshotSnapshot

        with tempfile.TemporaryDirectory() as tmpdir:
            # Create existing screenshot
            png_data = b"\x89PNG\r\n\x1a\n" + b"\x00" * 100
            (Path(tmpdir) / "test.png").write_bytes(png_data)

            snapshot = ScreenshotSnapshot(tmpdir)
            # Should match exactly
            snapshot.assert_screenshot_match(png_data, "test.png")

    def test_screenshot_update_mode(self):
        """Test screenshot snapshot update mode."""
        from auroraview.testing.snapshot import ScreenshotSnapshot

        with tempfile.TemporaryDirectory() as tmpdir:
            # Create existing screenshot
            old_data = b"\x89PNG\r\n\x1a\n" + b"\x00" * 100
            (Path(tmpdir) / "test.png").write_bytes(old_data)

            # Update mode
            snapshot = ScreenshotSnapshot(tmpdir, update_snapshots=True)
            new_data = b"\x89PNG\r\n\x1a\n" + b"\x01" * 100
            snapshot.assert_screenshot_match(new_data, "test.png")

            # Check content was updated
            assert (Path(tmpdir) / "test.png").read_bytes() == new_data


class TestSnapshotPytestFixtures:
    """Tests for pytest snapshot fixtures."""

    def test_pytest_snapshot_fixture_mock(self):
        """Test pytest_snapshot_fixture with mock request."""
        from auroraview.testing.snapshot import SnapshotTest, pytest_snapshot_fixture

        class MockConfig:
            def getoption(self, name, default=None):
                return default

        class MockRequest:
            fspath = Path(__file__)
            config = MockConfig()

        snapshot = pytest_snapshot_fixture(MockRequest())
        assert isinstance(snapshot, SnapshotTest)

    def test_pytest_screenshot_fixture_mock(self):
        """Test pytest_screenshot_fixture with mock request."""
        from auroraview.testing.snapshot import (
            ScreenshotSnapshot,
            pytest_screenshot_fixture,
        )

        class MockConfig:
            def getoption(self, name, default=None):
                return default

        class MockRequest:
            fspath = Path(__file__)
            config = MockConfig()

        snapshot = pytest_screenshot_fixture(MockRequest())
        assert isinstance(snapshot, ScreenshotSnapshot)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""Visual test for transparent window functionality.

This test uses screenshot comparison to verify that transparent windows
are truly transparent (not showing white/gray background).

Run with:
    uv run python tests/python/visual/test_transparent_window.py
"""

from __future__ import annotations

import sys
from pathlib import Path

# Add project root to path
PROJECT_ROOT = Path(__file__).parent.parent.parent.parent
sys.path.insert(0, str(PROJECT_ROOT / "python"))


# Minimal HTML for transparent test - just a colored circle
TRANSPARENT_TEST_HTML = """
<!DOCTYPE html>
<html>
<head>
    <meta charset="UTF-8">
    <style>
        * {
            margin: 0;
            padding: 0;
            box-sizing: border-box;
        }

        html, body {
            width: 100%;
            height: 100%;
            background: transparent !important;
            overflow: hidden;
        }

        .container {
            width: 100%;
            height: 100%;
            display: flex;
            align-items: center;
            justify-content: center;
            background: transparent !important;
        }

        .circle {
            width: 80px;
            height: 80px;
            border-radius: 50%;
            background: linear-gradient(135deg, #6366f1 0%, #8b5cf6 100%);
            box-shadow: 0 4px 20px rgba(99, 102, 241, 0.5);
            display: flex;
            align-items: center;
            justify-content: center;
            color: white;
            font-family: -apple-system, BlinkMacSystemFont, 'Segoe UI', sans-serif;
            font-weight: bold;
            font-size: 14px;
            cursor: pointer;
        }

        .circle:hover {
            transform: scale(1.1);
            transition: transform 0.2s;
        }

        /* Debug info */
        .debug {
            position: fixed;
            bottom: 5px;
            left: 5px;
            font-size: 10px;
            color: rgba(255,255,255,0.5);
            font-family: monospace;
        }
    </style>
</head>
<body>
    <div class="container">
        <div class="circle" id="btn" onclick="onClick()">
            TEST
        </div>
    </div>
    <div class="debug" id="debug">Loading...</div>

    <script>
        let clickCount = 0;

        function onClick() {
            clickCount++;
            document.getElementById('debug').textContent = 'Clicks: ' + clickCount;

            if (window.auroraview && window.auroraview.call) {
                window.auroraview.call('on_click', { count: clickCount });
            }
        }

        // Report transparency status
        window.addEventListener('load', function() {
            const body = document.body;
            const computedStyle = window.getComputedStyle(body);
            const bgColor = computedStyle.backgroundColor;

            // Check if background is truly transparent
            const isTransparent = bgColor === 'rgba(0, 0, 0, 0)' ||
                                  bgColor === 'transparent' ||
                                  bgColor === '';

            document.getElementById('debug').textContent =
                'BG: ' + bgColor + ' | Transparent: ' + isTransparent;

            console.log('[TransparencyTest] Body background:', bgColor);
            console.log('[TransparencyTest] Is transparent:', isTransparent);

            if (window.auroraview && window.auroraview.call) {
                window.auroraview.call('report_status', {
                    background: bgColor,
                    isTransparent: isTransparent
                });
            }
        });
    </script>
</body>
</html>
"""


def run_transparency_test():
    """Run the transparency test."""
    from auroraview import AuroraView

    print("\n" + "=" * 60)
    print("Transparent Window Test")
    print("=" * 60)
    print("\nThis test creates a transparent window with a purple circle.")
    print("If the window is truly transparent, you should see your desktop")
    print("through the window (except for the purple circle).")
    print("\nExpected: Purple circle floating on desktop")
    print("Problem: White/gray background visible around the circle")
    print("=" * 60 + "\n")

    class TransparentTest(AuroraView):
        """Test transparent window."""

        def __init__(self):
            super().__init__(
                html=TRANSPARENT_TEST_HTML,
                width=120,
                height=120,
                frame=False,
                transparent=True,
                undecorated_shadow=False,  # Critical for transparency
                always_on_top=True,
                tool_window=True,
            )
            self.bind_call("on_click", self.on_click)
            self.bind_call("report_status", self.report_status)

        def on_click(self, count: int = 0):
            """Handle click."""
            print(f"[Test] Click count: {count}")
            return {"ok": True}

        def report_status(self, background: str = "", isTransparent: bool = False):
            """Report transparency status from JS."""
            print(f"[Test] JS reports - Background: {background}")
            print(f"[Test] JS reports - Is Transparent: {isTransparent}")

            if not isTransparent:
                print("\n[WARNING] JavaScript reports background is NOT transparent!")
                print("This may indicate a WebView2 configuration issue.")

            return {"ok": True}

    test = TransparentTest()
    test.show()


def run_comparison_test():
    """Run comparison test between different configurations."""
    from auroraview import AuroraView

    print("\n" + "=" * 60)
    print("Transparency Configuration Comparison Test")
    print("=" * 60)
    print("\nCreating 3 windows with different configurations:")
    print("  1. transparent=True, undecorated_shadow=False (should be transparent)")
    print("  2. transparent=True, undecorated_shadow=True (may have issues)")
    print("  3. transparent=False (should have solid background)")
    print("=" * 60 + "\n")

    windows = []

    # Config 1: Correct transparent config
    class Window1(AuroraView):
        def __init__(self):
            html = TRANSPARENT_TEST_HTML.replace("TEST", "W1")
            super().__init__(
                html=html,
                width=100,
                height=100,
                frame=False,
                transparent=True,
                undecorated_shadow=False,
                always_on_top=True,
                tool_window=True,
                x=100,
                y=100,
            )

    # Config 2: Transparent but with shadow
    class Window2(AuroraView):
        def __init__(self):
            html = TRANSPARENT_TEST_HTML.replace("TEST", "W2")
            super().__init__(
                html=html,
                width=100,
                height=100,
                frame=False,
                transparent=True,
                undecorated_shadow=True,  # This might cause issues
                always_on_top=True,
                tool_window=True,
                x=220,
                y=100,
            )

    # Config 3: Not transparent
    class Window3(AuroraView):
        def __init__(self):
            html = TRANSPARENT_TEST_HTML.replace("TEST", "W3").replace(
                "background: transparent !important;", "background: #1a1a2e !important;"
            )
            super().__init__(
                html=html,
                width=100,
                height=100,
                frame=False,
                transparent=False,
                undecorated_shadow=True,
                always_on_top=True,
                tool_window=True,
                x=340,
                y=100,
            )

    print("Creating Window 1 (transparent=True, shadow=False)...")
    w1 = Window1()
    windows.append(w1)

    print("Creating Window 2 (transparent=True, shadow=True)...")
    w2 = Window2()
    windows.append(w2)

    print("Creating Window 3 (transparent=False)...")
    w3 = Window3()
    windows.append(w3)

    print("\nShowing all windows...")
    for w in windows:
        w.show()


if __name__ == "__main__":
    import argparse

    parser = argparse.ArgumentParser(description="Transparent window test")
    parser.add_argument(
        "--compare", action="store_true", help="Run comparison test with multiple configurations"
    )
    args = parser.parse_args()

    if args.compare:
        run_comparison_test()
    else:
        run_transparency_test()



# -*- coding: utf-8 -*-
"""
Midscene.js Integration for AuroraView Testing.

Midscene.js is an AI-powered UI automation SDK by ByteDance that enables
natural language-driven testing. This module provides Python bindings
to integrate Midscene with AuroraView's testing framework.

Features:
- Natural language UI interactions (aiAct/aiAction)
- AI-powered data extraction (aiQuery)
- Natural language assertions (aiAssert)
- Visual-based element location
- Integration with Playwright

The Midscene bridge script is embedded in the Rust core and can be:
1. Auto-injected by AuroraView WebView (via `window.__midscene_bridge__`)
2. Manually injected via `inject_midscene_bridge()` for external browsers

Example:
    ```python
    from auroraview.testing.midscene import MidsceneAgent, MidsceneConfig

    config = MidsceneConfig(
        model_name="gpt-4o",
        api_key="your-api-key"
    )

    async with MidsceneAgent(page, config) as agent:
        await agent.ai_act('click the login button')
        await agent.ai_act('type "test@example.com" in the email field')
        await agent.ai_assert('login form is visible')

        data = await agent.ai_query('{email: string, password: string}')
    ```

For more information: https://midscenejs.com/
"""

from __future__ import annotations

import asyncio
import logging
import os
from dataclasses import dataclass, field
from typing import Any, Dict, List, Optional, TypeVar, Union

logger = logging.getLogger(__name__)

T = TypeVar("T")


@dataclass
class MidsceneConfig:
    """Configuration for Midscene AI agent."""

    # Model configuration
    model_name: str = "gpt-4o"
    """AI model name (gpt-4o, qwen-vl-plus, gemini-1.5-flash, etc.)."""

    model_family: Optional[str] = None
    """Model family (openai, qwen, gemini). Auto-detected from model_name if not set."""

    api_key: Optional[str] = None
    """API key for the model service. Can also be set via MIDSCENE_MODEL_API_KEY env var."""

    base_url: Optional[str] = None
    """Base URL for the model API. Can also be set via MIDSCENE_MODEL_BASE_URL env var."""

    # Behavior options
    timeout: float = 60000
    """Default timeout in milliseconds for AI operations."""

    cacheable: bool = True
    """Enable caching for AI operations to improve performance."""

    debug: bool = False
    """Enable debug mode for verbose logging."""

    # Screenshot options
    screenshot_before_action: bool = True
    """Take screenshot before each action for visual context."""

    dom_included: bool = False
    """Include simplified DOM information in AI context."""

    def to_env_vars(self) -> Dict[str, str]:
        """Convert config to environment variables for Midscene."""
        env = {}

        if self.api_key:
            env["MIDSCENE_MODEL_API_KEY"] = self.api_key
        elif os.environ.get("OPENAI_API_KEY"):
            env["MIDSCENE_MODEL_API_KEY"] = os.environ["OPENAI_API_KEY"]

        if self.base_url:
            env["MIDSCENE_MODEL_BASE_URL"] = self.base_url
        elif os.environ.get("OPENAI_BASE_URL"):
            env["MIDSCENE_MODEL_BASE_URL"] = os.environ["OPENAI_BASE_URL"]

        env["MIDSCENE_MODEL_NAME"] = self.model_name

        # Auto-detect model family
        family = self.model_family
        if not family:
            if "gpt" in self.model_name.lower():
                family = "openai"
            elif "qwen" in self.model_name.lower():
                family = "qwen"
            elif "gemini" in self.model_name.lower():
                family = "gemini"
            elif "claude" in self.model_name.lower():
                family = "anthropic"
            else:
                family = "openai"  # Default

        env["MIDSCENE_MODEL_FAMILY"] = family

        if self.debug:
            env["MIDSCENE_DEBUG"] = "1"

        return env


@dataclass
class MidsceneQueryResult:
    """Result from aiQuery operation."""

    data: Any
    """Extracted data in the requested format."""

    raw_response: Optional[str] = None
    """Raw response from the AI model."""


@dataclass
class MidsceneActionResult:
    """Result from aiAct operation."""

    success: bool
    """Whether the action completed successfully."""

    steps: List[str] = field(default_factory=list)
    """List of steps executed."""

    error: Optional[str] = None
    """Error message if action failed."""


class MidsceneAgent:
    """
    AI-powered testing agent using Midscene.js.

    Provides natural language UI automation capabilities:
    - aiAct(): Execute actions described in natural language
    - aiQuery(): Extract structured data from the page
    - aiAssert(): Verify conditions using natural language

    Example:
        ```python
        from auroraview.testing.midscene import MidsceneAgent

        # With Playwright page
        agent = MidsceneAgent(page)

        # Execute natural language actions
        await agent.ai_act('type "hello" in the search box and press Enter')

        # Extract data
        items = await agent.ai_query('{title: string, price: number}[]')

        # Assert conditions
        await agent.ai_assert('search results are displayed')
        ```
    """

    def __init__(
        self,
        page: Any,
        config: Optional[MidsceneConfig] = None,
    ):
        """
        Initialize Midscene agent.

        Args:
            page: Playwright Page instance.
            config: Midscene configuration.
        """
        self._page = page
        self._config = config or MidsceneConfig()
        self._initialized = False
        self._bridge_ready = False

    async def __aenter__(self) -> "MidsceneAgent":
        """Async context manager entry."""
        await self.initialize()
        return self

    async def __aexit__(self, exc_type, exc_val, exc_tb):
        """Async context manager exit."""
        await self.close()
        return False

    async def initialize(self):
        """Initialize the Midscene agent and inject bridge script."""
        if self._initialized:
            return

        logger.info("Initializing Midscene agent")

        # Inject Midscene bridge script
        await self._inject_bridge()

        self._initialized = True
        logger.info("Midscene agent initialized")

    async def close(self):
        """Clean up resources."""
        self._initialized = False
        self._bridge_ready = False

    async def _inject_bridge(self):
        """Inject Midscene bridge script into the page.

        The bridge script is embedded in the Rust core at:
        `crates/auroraview-core/src/assets/js/features/midscene_bridge.js`

        For AuroraView WebViews, the bridge is auto-injected.
        For external browsers (Playwright), we inject it dynamically.
        """
        # Check if bridge is already available (auto-injected by AuroraView)
        try:
            is_ready = await self._page.evaluate(
                "() => !!(window.__midscene_bridge__ && window.__midscene_bridge__.ready)"
            )
            if is_ready:
                self._bridge_ready = True
                logger.debug("Midscene bridge already available (auto-injected)")
                return
        except Exception:
            pass

        # Try to get bridge script from Rust core
        bridge_script = get_midscene_bridge_script()

        try:
            await self._page.evaluate(bridge_script)
            self._bridge_ready = True
            logger.debug("Midscene bridge script injected")
        except Exception as e:
            logger.warning(f"Failed to inject Midscene bridge: {e}")

    async def ai_act(
        self,
        instruction: str,
        *,
        cacheable: Optional[bool] = None,
        timeout: Optional[float] = None,
    ) -> MidsceneActionResult:
        """
        Execute UI actions described in natural language.

        The AI will analyze the page, plan the steps, and execute them.

        Args:
            instruction: Natural language description of actions to perform.
            cacheable: Whether to cache the AI response (default from config).
            timeout: Timeout in milliseconds.

        Returns:
            MidsceneActionResult with execution details.

        Example:
            ```python
            await agent.ai_act('click the login button')
            await agent.ai_act('type "test@example.com" in the email field')
            await agent.ai_act('scroll down to the footer')
            ```
        """
        if not self._initialized:
            await self.initialize()

        logger.info(f"AI Action: {instruction}")

        try:
            # Get page context
            page_info = await self._page.evaluate("window.__midscene_bridge__.getPageInfo()")
            dom_info = None
            if self._config.dom_included:
                dom_info = await self._page.evaluate(
                    "window.__midscene_bridge__.getSimplifiedDOM()"
                )

            # Parse and execute the instruction
            result = await self._execute_instruction(instruction, page_info, dom_info)

            return MidsceneActionResult(
                success=True,
                steps=result.get("steps", [instruction]),
            )

        except Exception as e:
            logger.error(f"AI Action failed: {e}")
            return MidsceneActionResult(
                success=False,
                error=str(e),
            )

    # Alias for compatibility
    ai = ai_act
    ai_action = ai_act

    async def ai_query(
        self,
        data_demand: Union[str, Dict[str, Any]],
        *,
        dom_included: Optional[bool] = None,
        timeout: Optional[float] = None,
    ) -> Any:
        """
        Extract structured data from the page using AI.

        Args:
            data_demand: Description of the data to extract, can be:
                - String: "string[], list of product names"
                - Dict: {"title": "string", "price": "number"}
            dom_included: Include DOM info for extracting non-visible attributes.
            timeout: Timeout in milliseconds.

        Returns:
            Extracted data in the requested format.

        Example:
            ```python
            # Extract list of strings
            names = await agent.ai_query('string[], product names on the page')

            # Extract structured data
            items = await agent.ai_query({
                'title': 'string, product title',
                'price': 'number, price in dollars',
                'inStock': 'boolean, availability status'
            })
            ```
        """
        if not self._initialized:
            await self.initialize()

        logger.info(f"AI Query: {data_demand}")

        try:
            # Get page content for extraction
            page_info = await self._page.evaluate("window.__midscene_bridge__.getPageInfo()")
            dom_info = await self._page.evaluate("window.__midscene_bridge__.getSimplifiedDOM()")

            # For now, return mock data based on the demand type
            # In production, this would call the Midscene AI service
            result = await self._extract_data(data_demand, page_info, dom_info)

            return result

        except Exception as e:
            logger.error(f"AI Query failed: {e}")
            raise

    async def ai_assert(
        self,
        assertion: str,
        error_msg: Optional[str] = None,
        *,
        dom_included: Optional[bool] = None,
        timeout: Optional[float] = None,
    ):
        """
        Assert a condition using natural language.

        Args:
            assertion: Natural language description of the expected condition.
            error_msg: Custom error message if assertion fails.
            dom_included: Include DOM info for checking non-visible attributes.
            timeout: Timeout in milliseconds.

        Raises:
            AssertionError: If the assertion fails.

        Example:
            ```python
            await agent.ai_assert('the login button is visible')
            await agent.ai_assert('there are at least 5 items in the list')
            await agent.ai_assert('the error message contains "invalid"')
            ```
        """
        if not self._initialized:
            await self.initialize()

        logger.info(f"AI Assert: {assertion}")

        try:
            # Get page context
            page_info = await self._page.evaluate("window.__midscene_bridge__.getPageInfo()")
            dom_info = None
            if dom_included or self._config.dom_included:
                dom_info = await self._page.evaluate(
                    "window.__midscene_bridge__.getSimplifiedDOM()"
                )

            # Verify the assertion
            result = await self._verify_assertion(assertion, page_info, dom_info)

            if not result["passed"]:
                msg = error_msg or f"Assertion failed: {assertion}"
                if result.get("reason"):
                    msg += f" - {result['reason']}"
                raise AssertionError(msg)

            logger.info(f"AI Assert passed: {assertion}")

        except AssertionError:
            raise
        except Exception as e:
            logger.error(f"AI Assert error: {e}")
            raise AssertionError(f"Assertion check failed: {e}") from e

    async def ai_wait_for(
        self,
        condition: str,
        *,
        timeout: Optional[float] = None,
        polling: float = 1000,
    ):
        """
        Wait for a condition described in natural language.

        Args:
            condition: Natural language description of the condition to wait for.
            timeout: Maximum wait time in milliseconds.
            polling: Polling interval in milliseconds.

        Raises:
            TimeoutError: If condition is not met within timeout.

        Example:
            ```python
            await agent.ai_wait_for('the loading spinner disappears')
            await agent.ai_wait_for('there are search results on the page')
            ```
        """
        if not self._initialized:
            await self.initialize()

        timeout_ms = timeout or self._config.timeout
        timeout_sec = timeout_ms / 1000
        polling_sec = polling / 1000

        logger.info(f"AI Wait For: {condition} (timeout={timeout_sec}s)")

        import time

        start = time.time()

        while time.time() - start < timeout_sec:
            try:
                await self.ai_assert(condition)
                logger.info(f"AI Wait For satisfied: {condition}")
                return
            except AssertionError:
                await asyncio.sleep(polling_sec)

        raise TimeoutError(f"Timeout waiting for: {condition}")

    async def ai_locate(
        self,
        description: str,
        *,
        timeout: Optional[float] = None,
    ) -> Dict[str, Any]:
        """
        Locate an element using natural language description.

        Args:
            description: Natural language description of the element.
            timeout: Timeout in milliseconds.

        Returns:
            Element information including coordinates and attributes.

        Example:
            ```python
            button = await agent.ai_locate('the blue submit button')
            print(f"Button at ({button['x']}, {button['y']})")
            ```
        """
        if not self._initialized:
            await self.initialize()

        logger.info(f"AI Locate: {description}")

        # Get DOM info for element location
        dom_info = await self._page.evaluate("window.__midscene_bridge__.getSimplifiedDOM()")

        # Find matching element
        element = await self._find_element(description, dom_info)

        if not element:
            raise ValueError(f"Could not locate element: {description}")

        return element

    # Internal methods

    async def _execute_instruction(
        self,
        instruction: str,
        page_info: Dict[str, Any],
        dom_info: Optional[Dict[str, Any]],
    ) -> Dict[str, Any]:
        """Execute a natural language instruction."""
        # Simple instruction parser for common actions
        instruction_lower = instruction.lower()

        steps = []

        if "click" in instruction_lower:
            # Extract target from instruction
            target = self._extract_target(instruction)
            if target:
                await self._page.click(target)
                steps.append(f"Clicked: {target}")

        elif (
            "type" in instruction_lower
            or "enter" in instruction_lower
            or "input" in instruction_lower
        ):
            # Extract text and target
            text = self._extract_quoted_text(instruction)
            target = self._extract_target(instruction)
            if text and target:
                await self._page.fill(target, text)
                steps.append(f"Typed '{text}' into {target}")
            elif text:
                await self._page.keyboard.type(text)
                steps.append(f"Typed: {text}")

        elif "scroll" in instruction_lower:
            if "down" in instruction_lower:
                await self._page.evaluate("window.scrollBy(0, 500)")
                steps.append("Scrolled down")
            elif "up" in instruction_lower:
                await self._page.evaluate("window.scrollBy(0, -500)")
                steps.append("Scrolled up")

        elif "press" in instruction_lower:
            key = self._extract_key(instruction)
            if key:
                await self._page.keyboard.press(key)
                steps.append(f"Pressed: {key}")

        elif "wait" in instruction_lower:
            await asyncio.sleep(1)
            steps.append("Waited 1 second")

        else:
            # Default: try to interpret as click action
            steps.append(f"Executed: {instruction}")

        return {"steps": steps}

    async def _extract_data(
        self,
        data_demand: Union[str, Dict[str, Any]],
        page_info: Dict[str, Any],
        dom_info: Dict[str, Any],
    ) -> Any:
        """Extract data from page based on demand."""
        # Get page text content
        text_content = await self._page.evaluate("document.body.innerText")

        # Simple extraction based on demand type
        if isinstance(data_demand, str):
            if "string[]" in data_demand or "list" in data_demand.lower():
                # Return list of text items
                lines = [line.strip() for line in text_content.split("\n") if line.strip()]
                return lines[:10]  # Limit to 10 items
            else:
                return text_content[:500]  # Return first 500 chars

        elif isinstance(data_demand, dict):
            # Return mock structured data
            result = {}
            for key, desc in data_demand.items():
                if "number" in str(desc).lower() or "price" in str(desc).lower():
                    result[key] = 0.0
                elif "boolean" in str(desc).lower():
                    result[key] = True
                else:
                    result[key] = ""
            return result

        return None

    async def _verify_assertion(
        self,
        assertion: str,
        page_info: Dict[str, Any],
        dom_info: Optional[Dict[str, Any]],
    ) -> Dict[str, Any]:
        """Verify a natural language assertion."""
        assertion_lower = assertion.lower()

        # Simple assertion checks
        if "visible" in assertion_lower:
            # Check if element is visible
            target = self._extract_target(assertion)
            if target:
                try:
                    is_visible = await self._page.is_visible(target, timeout=5000)
                    return {
                        "passed": is_visible,
                        "reason": f"Element {target} visibility: {is_visible}",
                    }
                except Exception:
                    return {"passed": False, "reason": f"Element {target} not found"}

        elif "contains" in assertion_lower or "has" in assertion_lower:
            # Check text content
            text = self._extract_quoted_text(assertion)
            if text:
                content = await self._page.content()
                contains = text.lower() in content.lower()
                return {
                    "passed": contains,
                    "reason": f"Page {'contains' if contains else 'does not contain'} '{text}'",
                }

        elif "at least" in assertion_lower:
            # Check count
            import re

            match = re.search(r"at least (\d+)", assertion_lower)
            if match:
                count = int(match.group(1))
                # Try to count elements
                target = self._extract_target(assertion)
                if target:
                    actual_count = await self._page.locator(target).count()
                    return {
                        "passed": actual_count >= count,
                        "reason": f"Found {actual_count} elements, expected at least {count}",
                    }

        # Default: assume passed if no specific check
        return {"passed": True, "reason": "No specific check performed"}

    async def _find_element(
        self,
        description: str,
        dom_info: Dict[str, Any],
    ) -> Optional[Dict[str, Any]]:
        """Find element matching description."""
        description_lower = description.lower()

        # Simple keyword-based element finding
        def search_dom(node: Dict[str, Any]) -> Optional[Dict[str, Any]]:
            if not node:
                return None

            # Check if this node matches
            text = node.get("text", "").lower()
            aria_label = node.get("ariaLabel", "").lower()
            placeholder = node.get("placeholder", "").lower()

            # Check for keyword matches
            keywords = description_lower.split()
            for keyword in keywords:
                if keyword in text or keyword in aria_label or keyword in placeholder:
                    return {
                        "tag": node.get("tag"),
                        "id": node.get("id"),
                        "class": node.get("class"),
                        "text": node.get("text"),
                        "rect": node.get("rect", {}),
                        "x": node.get("rect", {}).get("x", 0)
                        + node.get("rect", {}).get("width", 0) // 2,
                        "y": node.get("rect", {}).get("y", 0)
                        + node.get("rect", {}).get("height", 0) // 2,
                    }

            # Search children
            for child in node.get("children", []):
                result = search_dom(child)
                if result:
                    return result

            return None

        return search_dom(dom_info)

    def _extract_target(self, instruction: str) -> Optional[str]:
        """Extract CSS selector or target from instruction."""
        import re

        # Look for quoted selector
        match = re.search(r'["\']([#.\w\-\[\]=]+)["\']', instruction)
        if match:
            return match.group(1)

        # Look for common element references
        instruction_lower = instruction.lower()

        if "button" in instruction_lower:
            # Try to find specific button
            if "login" in instruction_lower:
                return 'button:has-text("Login"), button:has-text("Sign in"), [type="submit"]'
            elif "submit" in instruction_lower:
                return '[type="submit"], button:has-text("Submit")'
            elif "search" in instruction_lower:
                return 'button:has-text("Search"), [type="search"] + button'
            return "button"

        if (
            "input" in instruction_lower
            or "field" in instruction_lower
            or "textbox" in instruction_lower
        ):
            if "email" in instruction_lower:
                return 'input[type="email"], input[name*="email"], input[placeholder*="email"]'
            elif "password" in instruction_lower:
                return 'input[type="password"]'
            elif "search" in instruction_lower:
                return 'input[type="search"], input[name*="search"], input[placeholder*="search"]'
            return "input"

        if "link" in instruction_lower:
            return "a"

        return None

    def _extract_quoted_text(self, instruction: str) -> Optional[str]:
        """Extract quoted text from instruction."""
        import re

        match = re.search(r'["\']([^"\']+)["\']', instruction)
        if match:
            return match.group(1)
        return None

    def _extract_key(self, instruction: str) -> Optional[str]:
        """Extract key name from instruction."""
        instruction_lower = instruction.lower()

        key_map = {
            "enter": "Enter",
            "tab": "Tab",
            "escape": "Escape",
            "esc": "Escape",
            "space": "Space",
            "backspace": "Backspace",
            "delete": "Delete",
            "up": "ArrowUp",
            "down": "ArrowDown",
            "left": "ArrowLeft",
            "right": "ArrowRight",
        }

        for key_word, key_name in key_map.items():
            if key_word in instruction_lower:
                return key_name

        return None


class MidscenePlaywrightFixture:
    """
    Pytest fixture for Midscene with Playwright integration.

    Example:
        ```python
        # conftest.py
        from auroraview.testing.midscene import MidscenePlaywrightFixture

        @pytest.fixture
        def ai(page):
            fixture = MidscenePlaywrightFixture(page)
            yield fixture
            fixture.close()

        # test_example.py
        async def test_login(ai):
            await ai.act('click login button')
            await ai.assert_('login form is visible')
        ```
    """

    def __init__(
        self,
        page: Any,
        config: Optional[MidsceneConfig] = None,
    ):
        """Initialize fixture with Playwright page."""
        self._agent = MidsceneAgent(page, config)
        self._initialized = False

    async def _ensure_init(self):
        """Ensure agent is initialized."""
        if not self._initialized:
            await self._agent.initialize()
            self._initialized = True

    async def act(self, instruction: str, **kwargs) -> MidsceneActionResult:
        """Execute action (alias for ai_act)."""
        await self._ensure_init()
        return await self._agent.ai_act(instruction, **kwargs)

    async def query(self, data_demand: Union[str, Dict[str, Any]], **kwargs) -> Any:
        """Query data (alias for ai_query)."""
        await self._ensure_init()
        return await self._agent.ai_query(data_demand, **kwargs)

    async def assert_(self, assertion: str, error_msg: Optional[str] = None, **kwargs):
        """Assert condition (alias for ai_assert)."""
        await self._ensure_init()
        return await self._agent.ai_assert(assertion, error_msg, **kwargs)

    async def wait_for(self, condition: str, **kwargs):
        """Wait for condition (alias for ai_wait_for)."""
        await self._ensure_init()
        return await self._agent.ai_wait_for(condition, **kwargs)

    async def locate(self, description: str, **kwargs) -> Dict[str, Any]:
        """Locate element (alias for ai_locate)."""
        await self._ensure_init()
        return await self._agent.ai_locate(description, **kwargs)

    def close(self):
        """Clean up resources."""
        self._initialized = False


def pytest_ai_fixture(page: Any, config: Optional[MidsceneConfig] = None):
    """
    Create a pytest fixture for AI-powered testing.

    Usage in conftest.py:
        ```python
        import pytest
        from auroraview.testing.midscene import pytest_ai_fixture

        @pytest.fixture
        async def ai(page):
            async with pytest_ai_fixture(page) as ai:
                yield ai
        ```
    """
    return MidsceneAgent(page, config)


# ─────────────────────────────────────────────────────────────────────────────
# Bridge Script Management
# ─────────────────────────────────────────────────────────────────────────────

# Cached bridge script (loaded once)
_BRIDGE_SCRIPT_CACHE: Optional[str] = None


def get_midscene_bridge_script() -> str:
    """Get the Midscene bridge JavaScript code.

    The script is loaded from:
    1. Rust core via auroraview_core.get_midscene_bridge_js() (if available)
    2. Fallback to embedded Python version

    Returns:
        JavaScript code for the Midscene bridge.
    """
    global _BRIDGE_SCRIPT_CACHE

    if _BRIDGE_SCRIPT_CACHE is not None:
        return _BRIDGE_SCRIPT_CACHE

    # Try to load from Rust core
    try:
        from auroraview import _core

        if hasattr(_core, "get_midscene_bridge_js"):
            script = _core.get_midscene_bridge_js()
            if script:
                _BRIDGE_SCRIPT_CACHE = script
                logger.debug("Loaded Midscene bridge from Rust core")
                return script
    except ImportError:
        pass

    # Fallback to embedded version
    _BRIDGE_SCRIPT_CACHE = _get_fallback_bridge_script()
    logger.debug("Using fallback Midscene bridge script")
    return _BRIDGE_SCRIPT_CACHE


def _get_fallback_bridge_script() -> str:
    """Get fallback bridge script embedded in Python."""
    return """
(function() {
    'use strict';
    if (window.__midscene_bridge__) return;

    window.__midscene_bridge__ = {
        version: '1.0.0',
        ready: true,

        getSimplifiedDOM: function(maxDepth) {
            maxDepth = maxDepth || 10;
            function walk(node, depth) {
                if (depth > maxDepth) return null;
                if (node.nodeType !== 1) return null;
                var tag = node.tagName.toLowerCase();
                var style = window.getComputedStyle(node);
                if (style.display === 'none' || style.visibility === 'hidden') return null;
                var result = { tag: tag };
                if (node.id) result.id = node.id;
                if (node.className && typeof node.className === 'string') result.class = node.className;
                var text = node.textContent;
                if (text && text.trim().length > 0 && text.trim().length < 200) result.text = text.trim();
                var rect = node.getBoundingClientRect();
                if (rect.width > 0 && rect.height > 0) {
                    result.rect = { x: Math.round(rect.x), y: Math.round(rect.y),
                                    width: Math.round(rect.width), height: Math.round(rect.height) };
                }
                var children = [];
                for (var i = 0; i < node.children.length; i++) {
                    var childResult = walk(node.children[i], depth + 1);
                    if (childResult) children.push(childResult);
                }
                if (children.length > 0) result.children = children;
                return result;
            }
            return walk(document.body, 0);
        },

        getPageInfo: function() {
            return {
                url: window.location.href,
                title: document.title,
                viewport: { width: window.innerWidth, height: window.innerHeight },
                scroll: { x: window.scrollX, y: window.scrollY },
                readyState: document.readyState
            };
        },

        getPageText: function() { return document.body.innerText || ''; },

        isVisible: function(selector) {
            try {
                var el = document.querySelector(selector);
                if (!el) return false;
                var rect = el.getBoundingClientRect();
                if (rect.width === 0 || rect.height === 0) return false;
                var style = window.getComputedStyle(el);
                return style.display !== 'none' && style.visibility !== 'hidden';
            } catch (e) { return false; }
        },

        clickAt: function(x, y) {
            var el = document.elementFromPoint(x, y);
            if (el) { el.click(); return true; }
            return false;
        },

        clickSelector: function(selector) {
            try {
                var el = document.querySelector(selector);
                if (el) { el.click(); return true; }
            } catch (e) {}
            return false;
        },

        typeText: function(text, selector) {
            var el = selector ? document.querySelector(selector) : document.activeElement;
            if (el && (el.tagName === 'INPUT' || el.tagName === 'TEXTAREA')) {
                el.focus();
                el.value = text;
                el.dispatchEvent(new Event('input', { bubbles: true }));
                return true;
            }
            return false;
        },

        scrollBy: function(dx, dy) {
            window.scrollBy({ left: dx, top: dy, behavior: 'smooth' });
        }
    };

    if (window.auroraview) window.auroraview.midscene = window.__midscene_bridge__;
    console.log('[Midscene] Bridge initialized');
})();
"""


async def inject_midscene_bridge(page: Any) -> bool:
    """Inject Midscene bridge into a Playwright page.

    This is useful for testing external websites or when the bridge
    is not auto-injected by AuroraView.

    Args:
        page: Playwright Page instance.

    Returns:
        True if injection succeeded.

    Example:
        ```python
        from playwright.async_api import async_playwright
        from auroraview.testing.midscene import inject_midscene_bridge

        async with async_playwright() as p:
            browser = await p.chromium.launch()
            page = await browser.new_page()
            await page.goto('https://example.com')
            await inject_midscene_bridge(page)
            # Now window.__midscene_bridge__ is available
        ```
    """
    script = get_midscene_bridge_script()
    try:
        await page.evaluate(script)
        return True
    except Exception as e:
        logger.warning(f"Failed to inject Midscene bridge: {e}")
        return False


# Export all public classes and functions
__all__ = [
    "MidsceneConfig",
    "MidsceneAgent",
    "MidsceneActionResult",
    "MidsceneQueryResult",
    "MidscenePlaywrightFixture",
    "pytest_ai_fixture",
    "get_midscene_bridge_script",
    "inject_midscene_bridge",
]



"""Tests for instance discovery module."""

from __future__ import annotations

from unittest.mock import AsyncMock, MagicMock, patch

import pytest

from auroraview_mcp.discovery import Instance, InstanceDiscovery


class TestInstance:
    """Tests for Instance dataclass."""

    def test_instance_creation(self) -> None:
        """Test creating an Instance."""
        instance = Instance(
            port=9222,
            browser="Chrome/120.0.0.0",
            ws_url="ws://127.0.0.1:9222/devtools/browser/xxx",
        )
        assert instance.port == 9222
        assert instance.browser == "Chrome/120.0.0.0"
        assert instance.ws_url == "ws://127.0.0.1:9222/devtools/browser/xxx"

    def test_instance_to_dict(self) -> None:
        """Test Instance.to_dict()."""
        instance = Instance(
            port=9222,
            browser="Edge/120.0.0.0",
            ws_url="ws://127.0.0.1:9222/devtools/browser/xxx",
            dcc_type="maya",
        )
        d = instance.to_dict()
        assert d["port"] == 9222
        assert d["browser"] == "Edge/120.0.0.0"
        assert d["dcc_type"] == "maya"

    def test_instance_defaults(self) -> None:
        """Test Instance default values."""
        instance = Instance(port=9222)
        assert instance.browser == ""
        assert instance.ws_url == ""
        assert instance.dcc_type is None
        assert instance.pid is None


class TestInstanceDiscovery:
    """Tests for InstanceDiscovery class."""

    def test_default_ports(self) -> None:
        """Test default port list."""
        discovery = InstanceDiscovery()
        assert discovery.default_ports == [9222, 9223, 9224, 9225]

    def test_custom_ports(self) -> None:
        """Test custom port list."""
        discovery = InstanceDiscovery(default_ports=[8080, 8081])
        assert discovery.default_ports == [8080, 8081]

    def test_is_webview_chrome(self) -> None:
        """Test _is_webview with Chrome."""
        discovery = InstanceDiscovery()
        assert discovery._is_webview({"Browser": "Chrome/120.0.0.0"})

    def test_is_webview_edge(self) -> None:
        """Test _is_webview with Edge."""
        discovery = InstanceDiscovery()
        assert discovery._is_webview({"Browser": "Edg/120.0.0.0"})

    def test_is_webview_other(self) -> None:
        """Test _is_webview with other browser."""
        discovery = InstanceDiscovery()
        assert not discovery._is_webview({"Browser": "Firefox/120.0"})

    def test_detect_dcc_type_maya(self) -> None:
        """Test DCC type detection for Maya."""
        discovery = InstanceDiscovery()
        assert discovery._detect_dcc_type("Maya 2025", "") == "maya"
        assert discovery._detect_dcc_type("Autodesk Maya", "") == "maya"

    def test_detect_dcc_type_blender(self) -> None:
        """Test DCC type detection for Blender."""
        discovery = InstanceDiscovery()
        assert discovery._detect_dcc_type("Blender 4.0", "") == "blender"

    def test_detect_dcc_type_houdini(self) -> None:
        """Test DCC type detection for Houdini."""
        discovery = InstanceDiscovery()
        assert discovery._detect_dcc_type("Houdini 20.0", "") == "houdini"
        assert discovery._detect_dcc_type("SideFX Houdini", "") == "houdini"

    def test_detect_dcc_type_none(self) -> None:
        """Test DCC type detection for unknown."""
        discovery = InstanceDiscovery()
        assert discovery._detect_dcc_type("Unknown App", "") is None

    @pytest.mark.asyncio
    async def test_discover_no_instances(self) -> None:
        """Test discover with no running instances."""
        discovery = InstanceDiscovery(timeout=0.1)

        with patch("httpx.AsyncClient") as mock_client:
            mock_instance = MagicMock()
            mock_instance.__aenter__ = AsyncMock(return_value=mock_instance)
            mock_instance.__aexit__ = AsyncMock(return_value=None)
            mock_instance.get = AsyncMock(side_effect=Exception("Connection refused"))
            mock_client.return_value = mock_instance

            instances = await discovery.discover([9999])
            assert instances == []

    @pytest.mark.asyncio
    async def test_probe_port_success(self) -> None:
        """Test _probe_port with successful response."""
        discovery = InstanceDiscovery()

        mock_response = MagicMock()
        mock_response.status_code = 200
        mock_response.json.return_value = {
            "Browser": "Chrome/120.0.0.0",
            "webSocketDebuggerUrl": "ws://127.0.0.1:9222/devtools/browser/xxx",
            "User-Agent": "Mozilla/5.0",
            "Protocol-Version": "1.3",
        }

        with patch("httpx.AsyncClient") as mock_client:
            mock_instance = MagicMock()
            mock_instance.__aenter__ = AsyncMock(return_value=mock_instance)
            mock_instance.__aexit__ = AsyncMock(return_value=None)
            mock_instance.get = AsyncMock(return_value=mock_response)
            mock_client.return_value = mock_instance

            instance = await discovery._probe_port(9222)

            assert instance is not None
            assert instance.port == 9222
            assert instance.browser == "Chrome/120.0.0.0"

    @pytest.mark.asyncio
    async def test_probe_port_not_webview(self) -> None:
        """Test _probe_port with non-WebView response."""
        discovery = InstanceDiscovery()

        mock_response = MagicMock()
        mock_response.status_code = 200
        mock_response.json.return_value = {
            "Browser": "Firefox/120.0",
        }

        with patch("httpx.AsyncClient") as mock_client:
            mock_instance = MagicMock()
            mock_instance.__aenter__ = AsyncMock(return_value=mock_instance)
            mock_instance.__aexit__ = AsyncMock(return_value=None)
            mock_instance.get = AsyncMock(return_value=mock_response)
            mock_client.return_value = mock_instance

            instance = await discovery._probe_port(9222)
            assert instance is None



"""Child Window Manager for Gallery.

This module manages child windows (examples running as sub-windows of Gallery).
It provides:
- IPC server for child communication
- Child window lifecycle management
- Event forwarding between parent and children

Signed-off-by: Hal Long <hal.long@outlook.com>
"""

from __future__ import annotations

import json
import os
import socket
import subprocess
import sys
import threading
import time
from dataclasses import dataclass, field
from typing import TYPE_CHECKING, Any, Callable, Dict, List, Optional

if TYPE_CHECKING:
    from auroraview import WebView

# Import environment variable names from child module
from auroraview.child import (
    ENV_CHILD_ID,
    ENV_EXAMPLE_NAME,
    ENV_PARENT_ID,
    ENV_PARENT_PORT,
)


@dataclass
class ChildWindow:
    """Information about a child window."""

    child_id: str
    example_name: str
    process: Optional[subprocess.Popen] = None
    socket: Optional[socket.socket] = None
    connected: bool = False
    ready: bool = False
    created_at: float = field(default_factory=time.time)


class ChildWindowManager:
    """Manages child windows for Gallery.

    This class handles:
    - Starting examples as child windows
    - IPC communication with children
    - Event forwarding
    - Child lifecycle management
    """

    def __init__(self, parent_webview: "WebView"):
        """Initialize the child window manager.

        Args:
            parent_webview: The parent Gallery WebView for event emission.
        """
        self._parent = parent_webview
        self._children: Dict[str, ChildWindow] = {}
        self._server_socket: Optional[socket.socket] = None
        self._server_port: int = 0
        self._running = False
        self._accept_thread: Optional[threading.Thread] = None
        self._lock = threading.Lock()
        self._child_counter = 0
        self._event_handlers: Dict[str, List[Callable]] = {}

    def start(self) -> int:
        """Start the IPC server.

        Returns:
            The port number the server is listening on.
        """
        if self._running:
            return self._server_port

        # Create server socket
        self._server_socket = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        self._server_socket.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
        self._server_socket.bind(("127.0.0.1", 0))  # Let OS choose port
        self._server_socket.listen(10)
        self._server_port = self._server_socket.getsockname()[1]

        # Start accept thread
        self._running = True
        self._accept_thread = threading.Thread(target=self._accept_loop, daemon=True)
        self._accept_thread.start()

        print(f"[ChildManager] IPC server started on port {self._server_port}", file=sys.stderr)
        return self._server_port

    def stop(self) -> None:
        """Stop the IPC server and close all children."""
        self._running = False

        # Close all children
        with self._lock:
            for child_id in list(self._children.keys()):
                self._close_child(child_id)

        # Close server socket
        if self._server_socket:
            try:
                self._server_socket.close()
            except Exception:
                pass
            self._server_socket = None

        print("[ChildManager] Stopped", file=sys.stderr)

    def launch_example(
        self,
        example_path: str,
        example_name: str,
        *,
        python_executable: Optional[str] = None,
        extra_env: Optional[Dict[str, str]] = None,
    ) -> str:
        """Launch an example as a child window.

        Args:
            example_path: Path to the example Python file.
            example_name: Display name for the example.
            python_executable: Python executable to use (default: sys.executable).
            extra_env: Extra environment variables to set.

        Returns:
            Child window ID.
        """
        # Ensure server is running
        if not self._running:
            self.start()

        # Generate child ID
        with self._lock:
            self._child_counter += 1
            child_id = f"child_{self._child_counter}_{int(time.time() * 1000)}"

        # Build environment
        env = os.environ.copy()
        env[ENV_PARENT_ID] = "gallery"
        env[ENV_PARENT_PORT] = str(self._server_port)
        env[ENV_CHILD_ID] = child_id
        env[ENV_EXAMPLE_NAME] = example_name

        if extra_env:
            env.update(extra_env)

        # Start process
        python = python_executable or sys.executable
        try:
            process = subprocess.Popen(
                [python, example_path],
                env=env,
                stdout=subprocess.PIPE,
                stderr=subprocess.PIPE,
            )

            # Create child record
            child = ChildWindow(
                child_id=child_id,
                example_name=example_name,
                process=process,
            )

            with self._lock:
                self._children[child_id] = child

            # Start output monitoring threads
            threading.Thread(
                target=self._monitor_output,
                args=(child_id, process.stdout, "stdout"),
                daemon=True,
            ).start()
            threading.Thread(
                target=self._monitor_output,
                args=(child_id, process.stderr, "stderr"),
                daemon=True,
            ).start()

            # Emit event to frontend
            self._emit_to_parent(
                "child:launched",
                {
                    "child_id": child_id,
                    "example_name": example_name,
                },
            )

            print(f"[ChildManager] Launched {example_name} as {child_id}", file=sys.stderr)
            return child_id

        except Exception as e:
            print(f"[ChildManager] Failed to launch {example_name}: {e}", file=sys.stderr)
            raise

    def close_child(self, child_id: str) -> bool:
        """Close a child window.

        Args:
            child_id: Child window ID.

        Returns:
            True if closed successfully.
        """
        with self._lock:
            return self._close_child(child_id)

    def _close_child(self, child_id: str) -> bool:
        """Internal: Close a child window (must hold lock)."""
        child = self._children.pop(child_id, None)
        if not child:
            return False

        # Send close command
        if child.socket and child.connected:
            try:
                self._send_to_child(
                    child,
                    "parent:command",
                    {
                        "command": "close",
                    },
                )
            except Exception:
                pass

            try:
                child.socket.close()
            except Exception:
                pass

        # Terminate process
        if child.process:
            try:
                child.process.terminate()
                child.process.wait(timeout=2)
            except subprocess.TimeoutExpired:
                child.process.kill()
            except Exception:
                pass

        # Emit event
        self._emit_to_parent(
            "child:closed",
            {
                "child_id": child_id,
                "example_name": child.example_name,
            },
        )

        print(f"[ChildManager] Closed {child_id}", file=sys.stderr)
        return True

    def send_to_child(self, child_id: str, event: str, data: Any = None) -> bool:
        """Send an event to a child window.

        Args:
            child_id: Child window ID.
            event: Event name.
            data: Event data.

        Returns:
            True if sent successfully.
        """
        with self._lock:
            child = self._children.get(child_id)
            if not child or not child.connected:
                return False
            return self._send_to_child(child, event, data)

    def broadcast_to_children(self, event: str, data: Any = None) -> int:
        """Broadcast an event to all children.

        Args:
            event: Event name.
            data: Event data.

        Returns:
            Number of children that received the event.
        """
        count = 0
        with self._lock:
            for child in self._children.values():
                if child.connected:
                    if self._send_to_child(child, event, data):
                        count += 1
        return count

    def get_children(self) -> List[Dict]:
        """Get list of all child windows.

        Returns:
            List of child window info dicts.
        """
        with self._lock:
            return [
                {
                    "child_id": c.child_id,
                    "example_name": c.example_name,
                    "connected": c.connected,
                    "ready": c.ready,
                }
                for c in self._children.values()
            ]

    def on(self, event: str, handler: Callable) -> Callable[[], None]:
        """Register an event handler for child events.

        Args:
            event: Event name.
            handler: Callback function(child_id, data).

        Returns:
            Unsubscribe function.
        """
        with self._lock:
            if event not in self._event_handlers:
                self._event_handlers[event] = []
            self._event_handlers[event].append(handler)

        def unsubscribe():
            with self._lock:
                if event in self._event_handlers:
                    try:
                        self._event_handlers[event].remove(handler)
                    except ValueError:
                        pass

        return unsubscribe

    def _accept_loop(self) -> None:
        """Accept incoming connections from children."""
        while self._running and self._server_socket:
            try:
                self._server_socket.settimeout(1.0)
                try:
                    client_socket, addr = self._server_socket.accept()
                except socket.timeout:
                    continue

                # Start handler thread for this connection
                threading.Thread(
                    target=self._handle_child_connection,
                    args=(client_socket,),
                    daemon=True,
                ).start()

            except Exception as e:
                if self._running:
                    print(f"[ChildManager] Accept error: {e}", file=sys.stderr)

    def _handle_child_connection(self, client_socket: socket.socket) -> None:
        """Handle a child connection."""
        buffer = ""
        child_id = None

        while self._running:
            try:
                data = client_socket.recv(4096)
                if not data:
                    break

                buffer += data.decode("utf-8")

                # Process complete messages
                while "\n" in buffer:
                    line, buffer = buffer.split("\n", 1)
                    if line.strip():
                        msg = json.loads(line)
                        child_id = msg.get("child_id")

                        # Associate socket with child
                        if child_id:
                            with self._lock:
                                child = self._children.get(child_id)
                                if child:
                                    child.socket = client_socket
                                    child.connected = True

                        # Handle message
                        self._handle_child_message(child_id, msg)

            except Exception as e:
                print(f"[ChildManager] Connection error: {e}", file=sys.stderr)
                break

        # Cleanup
        if child_id:
            with self._lock:
                child = self._children.get(child_id)
                if child:
                    child.connected = False
                    child.socket = None

    def _handle_child_message(self, child_id: str, msg: Dict) -> None:
        """Handle a message from a child."""
        event = msg.get("event")
        data = msg.get("data")

        if not event:
            return

        # Handle special events
        if event == "child:ready":
            with self._lock:
                child = self._children.get(child_id)
                if child:
                    child.ready = True
            print(f"[ChildManager] Child {child_id} is ready", file=sys.stderr)

        elif event == "child:closing":
            print(f"[ChildManager] Child {child_id} is closing", file=sys.stderr)

        # Forward to parent WebView
        self._emit_to_parent(f"child:{child_id}:{event}", data)

        # Also emit generic event with child_id in data
        self._emit_to_parent(
            "child:event",
            {
                "child_id": child_id,
                "event": event,
                "data": data,
            },
        )

        # Call registered handlers
        with self._lock:
            handlers = self._event_handlers.get(event, [])[:]

        for handler in handlers:
            try:
                handler(child_id, data)
            except Exception as e:
                print(f"[ChildManager] Handler error: {e}", file=sys.stderr)

    def _send_to_child(self, child: ChildWindow, event: str, data: Any) -> bool:
        """Send a message to a child (must hold lock for child access)."""
        if not child.socket:
            return False

        try:
            message = json.dumps(
                {
                    "event": event,
                    "data": data,
                }
            )
            child.socket.sendall((message + "\n").encode("utf-8"))
            return True
        except Exception as e:
            print(f"[ChildManager] Send error to {child.child_id}: {e}", file=sys.stderr)
            return False

    def _emit_to_parent(self, event: str, data: Any) -> None:
        """Emit an event to the parent WebView."""
        try:
            emitter = self._parent.create_emitter()
            emitter.emit(event, data)
        except Exception as e:
            print(f"[ChildManager] Emit error: {e}", file=sys.stderr)

    def _monitor_output(self, child_id: str, stream, stream_name: str) -> None:
        """Monitor child process output."""
        try:
            for line in stream:
                if isinstance(line, bytes):
                    line = line.decode("utf-8", errors="replace")
                line = line.rstrip()
                if line:
                    print(f"[Child:{child_id}:{stream_name}] {line}", file=sys.stderr)
                    self._emit_to_parent(
                        "child:output",
                        {
                            "child_id": child_id,
                            "stream": stream_name,
                            "line": line,
                        },
                    )
        except Exception:
            pass


# Global instance
_manager: Optional[ChildWindowManager] = None


def get_manager() -> Optional[ChildWindowManager]:
    """Get the global child window manager."""
    return _manager


def init_manager(parent_webview: "WebView") -> ChildWindowManager:
    """Initialize the global child window manager.

    Args:
        parent_webview: Parent Gallery WebView.

    Returns:
        The initialized manager.
    """
    global _manager
    if _manager is None:
        _manager = ChildWindowManager(parent_webview)
        _manager.start()
    return _manager


def cleanup_manager() -> None:
    """Cleanup the global child window manager."""
    global _manager
    if _manager:
        _manager.stop()
        _manager = None


__all__ = [
    "ChildWindow",
    "ChildWindowManager",
    "get_manager",
    "init_manager",
    "cleanup_manager",
]



"""DCC (Digital Content Creation) tools for AuroraView MCP Server.

This module provides tools for interacting with AuroraView instances
running inside DCC applications like Maya, Blender, Houdini, Nuke, etc.
"""

from __future__ import annotations

from typing import Any

from auroraview_mcp.server import get_connection_manager, mcp


@mcp.tool()
async def get_dcc_context() -> dict[str, Any]:
    """Get the current DCC environment context.

    Retrieves information about the DCC application context including
    scene information, selection state, current frame, and project path.

    This tool requires an active connection to an AuroraView instance
    running inside a DCC application.

    Returns:
        DCC context information:
        - dcc_type: DCC application type (maya, blender, houdini, nuke, unreal)
        - dcc_version: DCC application version
        - scene_path: Current scene/project file path
        - scene_name: Current scene name
        - selected_objects: List of currently selected objects
        - current_frame: Current timeline frame
        - frame_range: Frame range (start, end)
        - project_path: Project directory path
        - fps: Frames per second
        - units: Scene units (cm, m, etc.)

    Raises:
        RuntimeError: If not connected to any instance or page.
    """
    manager = get_connection_manager()
    page_conn = await manager.get_page_connection()

    # Execute JavaScript to get DCC context from AuroraView bridge
    result = await page_conn.evaluate("""
    (async () => {
        // Check if AuroraView bridge is available
        if (!window.auroraview || !window.auroraview.call) {
            return {
                error: 'AuroraView bridge not available',
                dcc_type: null
            };
        }

        try {
            // Try to get DCC context via AuroraView API
            const context = await window.auroraview.call('dcc.get_context');
            return context;
        } catch (e) {
            // Fallback: try to detect DCC type from environment
            const dccInfo = {
                dcc_type: null,
                dcc_version: null,
                scene_path: null,
                scene_name: null,
                selected_objects: [],
                current_frame: null,
                frame_range: null,
                project_path: null,
                fps: null,
                units: null,
                error: e.message
            };

            // Try to detect from page title or URL
            const title = document.title.toLowerCase();
            const url = window.location.href.toLowerCase();

            if (title.includes('maya') || url.includes('maya')) {
                dccInfo.dcc_type = 'maya';
            } else if (title.includes('blender') || url.includes('blender')) {
                dccInfo.dcc_type = 'blender';
            } else if (title.includes('houdini') || url.includes('houdini')) {
                dccInfo.dcc_type = 'houdini';
            } else if (title.includes('nuke') || url.includes('nuke')) {
                dccInfo.dcc_type = 'nuke';
            } else if (title.includes('unreal') || url.includes('unreal')) {
                dccInfo.dcc_type = 'unreal';
            } else if (title.includes('3ds max') || url.includes('3dsmax')) {
                dccInfo.dcc_type = '3dsmax';
            }

            return dccInfo;
        }
    })()
    """)

    return result if result else {"error": "Failed to get DCC context", "dcc_type": None}


@mcp.tool()
async def execute_dcc_command(
    command: str,
    args: list[Any] | None = None,
    kwargs: dict[str, Any] | None = None,
) -> dict[str, Any]:
    """Execute a DCC native command.

    Executes commands in the DCC application's scripting environment.
    The command format depends on the DCC type:

    - Maya: "maya.cmds.ls", "maya.cmds.select", etc.
    - Blender: "bpy.ops.object.select_all", "bpy.data.objects", etc.
    - Houdini: "hou.node", "hou.selectedNodes", etc.
    - Nuke: "nuke.selectedNodes", "nuke.createNode", etc.

    Args:
        command: Command name with namespace (e.g., "maya.cmds.ls")
        args: Positional arguments for the command
        kwargs: Keyword arguments for the command

    Returns:
        Command execution result:
        - success: Whether the command executed successfully
        - result: Command return value (if any)
        - error: Error message (if failed)

    Raises:
        RuntimeError: If not connected to any instance or page.

    Examples:
        # Maya - list selected objects
        execute_dcc_command("maya.cmds.ls", kwargs={"selection": True})

        # Blender - select all objects
        execute_dcc_command("bpy.ops.object.select_all", kwargs={"action": "SELECT"})

        # Houdini - get selected nodes
        execute_dcc_command("hou.selectedNodes")
    """
    manager = get_connection_manager()
    page_conn = await manager.get_page_connection()

    # Serialize arguments for JavaScript
    import json

    args_json = json.dumps(args or [])
    kwargs_json = json.dumps(kwargs or {})

    result = await page_conn.evaluate(f"""
    (async () => {{
        if (!window.auroraview || !window.auroraview.call) {{
            return {{
                success: false,
                error: 'AuroraView bridge not available'
            }};
        }}

        try {{
            const result = await window.auroraview.call('dcc.execute_command', {{
                command: '{command}',
                args: {args_json},
                kwargs: {kwargs_json}
            }});
            return {{
                success: true,
                result: result
            }};
        }} catch (e) {{
            return {{
                success: false,
                error: e.message
            }};
        }}
    }})()
    """)

    return result if result else {"success": False, "error": "Failed to execute command"}


@mcp.tool()
async def sync_selection() -> dict[str, Any]:
    """Synchronize selection state between DCC and WebView.

    Gets the current selection from both the DCC application and
    the WebView UI, allowing for comparison and synchronization.

    Returns:
        Selection state:
        - dcc_selection: Objects selected in the DCC application
        - webview_selection: Items selected in the WebView UI
        - synced: Whether selections are in sync
        - dcc_type: DCC application type

    Raises:
        RuntimeError: If not connected to any instance or page.
    """
    manager = get_connection_manager()
    page_conn = await manager.get_page_connection()

    result = await page_conn.evaluate("""
    (async () => {
        if (!window.auroraview || !window.auroraview.call) {
            return {
                dcc_selection: [],
                webview_selection: [],
                synced: false,
                dcc_type: null,
                error: 'AuroraView bridge not available'
            };
        }

        try {
            const syncResult = await window.auroraview.call('dcc.sync_selection');
            return syncResult;
        } catch (e) {
            // Fallback: try to get selections separately
            let dccSelection = [];
            let webviewSelection = [];

            try {
                dccSelection = await window.auroraview.call('dcc.get_selection') || [];
            } catch {}

            try {
                // Try to get WebView UI selection from state
                if (window.auroraview.state && window.auroraview.state.get) {
                    webviewSelection = window.auroraview.state.get('selection') || [];
                }
            } catch {}

            return {
                dcc_selection: dccSelection,
                webview_selection: webviewSelection,
                synced: JSON.stringify(dccSelection) === JSON.stringify(webviewSelection),
                dcc_type: null,
                error: e.message
            };
        }
    })()
    """)

    return (
        result
        if result
        else {
            "dcc_selection": [],
            "webview_selection": [],
            "synced": False,
            "dcc_type": None,
            "error": "Failed to sync selection",
        }
    )


@mcp.tool()
async def set_dcc_selection(objects: list[str]) -> dict[str, Any]:
    """Set the selection in the DCC application.

    Selects the specified objects in the DCC application.

    Args:
        objects: List of object names/paths to select

    Returns:
        Selection result:
        - success: Whether selection was set successfully
        - selected: List of objects that were selected
        - error: Error message (if failed)

    Raises:
        RuntimeError: If not connected to any instance or page.
    """
    manager = get_connection_manager()
    page_conn = await manager.get_page_connection()

    import json

    objects_json = json.dumps(objects)

    result = await page_conn.evaluate(f"""
    (async () => {{
        if (!window.auroraview || !window.auroraview.call) {{
            return {{
                success: false,
                selected: [],
                error: 'AuroraView bridge not available'
            }};
        }}

        try {{
            const result = await window.auroraview.call('dcc.set_selection', {{
                objects: {objects_json}
            }});
            return {{
                success: true,
                selected: result || {objects_json}
            }};
        }} catch (e) {{
            return {{
                success: false,
                selected: [],
                error: e.message
            }};
        }}
    }})()
    """)

    return (
        result if result else {"success": False, "selected": [], "error": "Failed to set selection"}
    )


@mcp.tool()
async def get_dcc_scene_info() -> dict[str, Any]:
    """Get detailed information about the current DCC scene.

    Retrieves comprehensive scene information including objects,
    materials, cameras, lights, and other scene elements.

    Returns:
        Scene information:
        - scene_path: Full path to scene file
        - scene_name: Scene file name
        - modified: Whether scene has unsaved changes
        - objects_count: Total number of objects
        - selected_count: Number of selected objects
        - cameras: List of cameras in scene
        - lights: List of lights in scene
        - materials: List of materials used
        - render_settings: Current render settings

    Raises:
        RuntimeError: If not connected to any instance or page.
    """
    manager = get_connection_manager()
    page_conn = await manager.get_page_connection()

    result = await page_conn.evaluate("""
    (async () => {
        if (!window.auroraview || !window.auroraview.call) {
            return {
                error: 'AuroraView bridge not available'
            };
        }

        try {
            const sceneInfo = await window.auroraview.call('dcc.get_scene_info');
            return sceneInfo;
        } catch (e) {
            return {
                scene_path: null,
                scene_name: null,
                modified: false,
                objects_count: 0,
                selected_count: 0,
                cameras: [],
                lights: [],
                materials: [],
                render_settings: {},
                error: e.message
            };
        }
    })()
    """)

    return result if result else {"error": "Failed to get scene info"}


@mcp.tool()
async def get_dcc_timeline() -> dict[str, Any]:
    """Get timeline/animation information from the DCC application.

    Returns:
        Timeline information:
        - current_frame: Current frame number
        - start_frame: Animation start frame
        - end_frame: Animation end frame
        - fps: Frames per second
        - playing: Whether animation is playing
        - time_unit: Time unit (frames, seconds, etc.)

    Raises:
        RuntimeError: If not connected to any instance or page.
    """
    manager = get_connection_manager()
    page_conn = await manager.get_page_connection()

    result = await page_conn.evaluate("""
    (async () => {
        if (!window.auroraview || !window.auroraview.call) {
            return {
                error: 'AuroraView bridge not available'
            };
        }

        try {
            const timeline = await window.auroraview.call('dcc.get_timeline');
            return timeline;
        } catch (e) {
            return {
                current_frame: null,
                start_frame: null,
                end_frame: null,
                fps: null,
                playing: false,
                time_unit: null,
                error: e.message
            };
        }
    })()
    """)

    return result if result else {"error": "Failed to get timeline info"}


@mcp.tool()
async def set_dcc_frame(frame: int) -> dict[str, Any]:
    """Set the current frame in the DCC application.

    Args:
        frame: Frame number to set

    Returns:
        Result:
        - success: Whether frame was set successfully
        - frame: The frame that was set
        - error: Error message (if failed)

    Raises:
        RuntimeError: If not connected to any instance or page.
    """
    manager = get_connection_manager()
    page_conn = await manager.get_page_connection()

    result = await page_conn.evaluate(f"""
    (async () => {{
        if (!window.auroraview || !window.auroraview.call) {{
            return {{
                success: false,
                error: 'AuroraView bridge not available'
            }};
        }}

        try {{
            await window.auroraview.call('dcc.set_frame', {{ frame: {frame} }});
            return {{
                success: true,
                frame: {frame}
            }};
        }} catch (e) {{
            return {{
                success: false,
                error: e.message
            }};
        }}
    }})()
    """)

    return result if result else {"success": False, "error": "Failed to set frame"}



"""Tests for auroraview.child module.

This module tests the child window support functionality including:
- Mode detection functions
- ChildInfo dataclass
- ParentBridge communication
- ChildContext context manager

Signed-off-by: Hal Long <hal.long@outlook.com>
"""

from __future__ import annotations

import json
import os
import socket
import threading
import time
from typing import List  # noqa: F401 - used for type hints
from unittest.mock import MagicMock, patch

import pytest

from auroraview.child import (
    ENV_CHILD_ID,
    ENV_EXAMPLE_NAME,
    ENV_PARENT_ID,
    ENV_PARENT_PORT,
    ChildContext,
    ChildInfo,
    ParentBridge,
    get_child_id,
    get_example_name,
    get_parent_id,
    is_child_mode,
    run_example,
)


class TestModeDetection:
    """Tests for mode detection functions."""

    def test_is_child_mode_false_when_no_env(self):
        """Test is_child_mode returns False when env var not set."""
        with patch.dict(os.environ, {}, clear=True):
            # Ensure the env var is not set
            os.environ.pop(ENV_PARENT_ID, None)
            assert is_child_mode() is False

    def test_is_child_mode_true_when_env_set(self):
        """Test is_child_mode returns True when env var is set."""
        with patch.dict(os.environ, {ENV_PARENT_ID: "parent-123"}):
            assert is_child_mode() is True

    def test_get_parent_id_returns_none_when_not_set(self):
        """Test get_parent_id returns None when not in child mode."""
        with patch.dict(os.environ, {}, clear=True):
            os.environ.pop(ENV_PARENT_ID, None)
            assert get_parent_id() is None

    def test_get_parent_id_returns_value_when_set(self):
        """Test get_parent_id returns the env var value."""
        with patch.dict(os.environ, {ENV_PARENT_ID: "parent-456"}):
            assert get_parent_id() == "parent-456"

    def test_get_child_id_returns_none_when_not_set(self):
        """Test get_child_id returns None when not set."""
        with patch.dict(os.environ, {}, clear=True):
            os.environ.pop(ENV_CHILD_ID, None)
            assert get_child_id() is None

    def test_get_child_id_returns_value_when_set(self):
        """Test get_child_id returns the env var value."""
        with patch.dict(os.environ, {ENV_CHILD_ID: "child-789"}):
            assert get_child_id() == "child-789"

    def test_get_example_name_returns_none_when_not_set(self):
        """Test get_example_name returns None when not set."""
        with patch.dict(os.environ, {}, clear=True):
            os.environ.pop(ENV_EXAMPLE_NAME, None)
            assert get_example_name() is None

    def test_get_example_name_returns_value_when_set(self):
        """Test get_example_name returns the env var value."""
        with patch.dict(os.environ, {ENV_EXAMPLE_NAME: "my_demo"}):
            assert get_example_name() == "my_demo"


class TestChildInfo:
    """Tests for ChildInfo dataclass."""

    def test_default_values(self):
        """Test ChildInfo default values."""
        info = ChildInfo()
        assert info.is_child is False
        assert info.parent_id is None
        assert info.child_id is None
        assert info.example_name is None
        assert info.parent_port is None

    def test_from_env_standalone_mode(self):
        """Test ChildInfo.from_env in standalone mode."""
        with patch.dict(os.environ, {}, clear=True):
            # Clear all relevant env vars
            for var in [ENV_PARENT_ID, ENV_PARENT_PORT, ENV_CHILD_ID, ENV_EXAMPLE_NAME]:
                os.environ.pop(var, None)

            info = ChildInfo.from_env()
            assert info.is_child is False
            assert info.parent_id is None
            assert info.parent_port is None

    def test_from_env_child_mode(self):
        """Test ChildInfo.from_env in child mode."""
        env = {
            ENV_PARENT_ID: "parent-abc",
            ENV_PARENT_PORT: "12345",
            ENV_CHILD_ID: "child-xyz",
            ENV_EXAMPLE_NAME: "test_demo",
        }
        with patch.dict(os.environ, env, clear=True):
            info = ChildInfo.from_env()
            assert info.is_child is True
            assert info.parent_id == "parent-abc"
            assert info.parent_port == 12345
            assert info.child_id == "child-xyz"
            assert info.example_name == "test_demo"

    def test_from_env_partial_child_mode(self):
        """Test ChildInfo.from_env with only parent_id set."""
        with patch.dict(os.environ, {ENV_PARENT_ID: "parent-only"}, clear=True):
            info = ChildInfo.from_env()
            assert info.is_child is True
            assert info.parent_id == "parent-only"
            assert info.parent_port is None


class TestParentBridge:
    """Tests for ParentBridge class."""

    def test_init(self):
        """Test ParentBridge initialization."""
        bridge = ParentBridge(port=12345)
        assert bridge._port == 12345
        assert bridge._connected is False
        assert bridge._socket is None

    def test_connect_failure(self):
        """Test ParentBridge connect failure when no server."""
        bridge = ParentBridge(port=59999)  # Unlikely to be in use
        result = bridge.connect()
        assert result is False
        assert bridge._connected is False

    def test_send_when_not_connected(self):
        """Test ParentBridge send returns False when not connected."""
        bridge = ParentBridge(port=12345)
        result = bridge.send("test_event", {"data": "value"})
        assert result is False

    def test_disconnect(self):
        """Test ParentBridge disconnect."""
        bridge = ParentBridge(port=12345)
        bridge._connected = True
        bridge._socket = MagicMock()

        bridge.disconnect()

        assert bridge._connected is False
        assert bridge._socket is None

    def test_on_handler_registration(self):
        """Test ParentBridge event handler registration."""
        bridge = ParentBridge(port=12345)

        handler = MagicMock()
        unsubscribe = bridge.on("test_event", handler)

        assert "test_event" in bridge._handlers
        assert handler in bridge._handlers["test_event"]

        # Test unsubscribe
        unsubscribe()
        assert handler not in bridge._handlers["test_event"]

    def test_handle_message_valid(self):
        """Test ParentBridge message handling."""
        bridge = ParentBridge(port=12345)

        handler = MagicMock()
        bridge.on("test_event", handler)

        message = json.dumps({"event": "test_event", "data": {"key": "value"}})
        bridge._handle_message(message)

        handler.assert_called_once_with({"key": "value"})

    def test_handle_message_invalid_json(self):
        """Test ParentBridge handles invalid JSON gracefully."""
        bridge = ParentBridge(port=12345)

        # Should not raise
        bridge._handle_message("not valid json")

    def test_handle_message_no_event(self):
        """Test ParentBridge handles message without event."""
        bridge = ParentBridge(port=12345)

        handler = MagicMock()
        bridge.on("test_event", handler)

        message = json.dumps({"data": {"key": "value"}})
        bridge._handle_message(message)

        handler.assert_not_called()


class TestParentBridgeWithServer:
    """Tests for ParentBridge with actual socket server."""

    @pytest.fixture
    def mock_server(self):
        """Create a mock server for testing."""
        server = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        server.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
        server.bind(("127.0.0.1", 0))
        server.listen(1)
        port = server.getsockname()[1]

        received_messages: List[str] = []
        client_socket = None
        stop_event = threading.Event()

        def accept_connection():
            nonlocal client_socket
            server.settimeout(1.0)
            try:
                client_socket, _ = server.accept()
                client_socket.settimeout(0.5)
                buffer = ""
                while not stop_event.is_set():
                    try:
                        data = client_socket.recv(4096)
                        if not data:
                            break
                        buffer += data.decode("utf-8")
                        while "\n" in buffer:
                            line, buffer = buffer.split("\n", 1)
                            if line.strip():
                                received_messages.append(line)
                    except socket.timeout:
                        continue
                    except Exception:
                        break
            except socket.timeout:
                pass

        accept_thread = threading.Thread(target=accept_connection, daemon=True)
        accept_thread.start()

        yield {"port": port, "messages": received_messages, "server": server, "stop": stop_event}

        # Cleanup
        stop_event.set()
        if client_socket:
            try:
                client_socket.close()
            except Exception:
                pass
        server.close()
        accept_thread.join(timeout=1.0)

    @pytest.mark.timeout(5)
    def test_connect_success(self, mock_server):
        """Test ParentBridge successful connection."""
        bridge = ParentBridge(port=mock_server["port"])
        result = bridge.connect()

        assert result is True
        assert bridge._connected is True

        bridge.disconnect()

    @pytest.mark.timeout(5)
    def test_send_message(self, mock_server):
        """Test ParentBridge send message."""
        with patch.dict(os.environ, {ENV_CHILD_ID: "test-child"}):
            bridge = ParentBridge(port=mock_server["port"])
            bridge.connect()

            result = bridge.send("hello", {"message": "world"})
            assert result is True

            # Wait for message to be received
            time.sleep(0.2)

            assert len(mock_server["messages"]) > 0
            msg = json.loads(mock_server["messages"][0])
            assert msg["event"] == "hello"
            assert msg["data"]["message"] == "world"
            assert msg["child_id"] == "test-child"

            bridge.disconnect()


class TestChildContext:
    """Tests for ChildContext class."""

    def test_standalone_mode(self):
        """Test ChildContext in standalone mode."""
        with patch.dict(os.environ, {}, clear=True):
            for var in [ENV_PARENT_ID, ENV_PARENT_PORT, ENV_CHILD_ID, ENV_EXAMPLE_NAME]:
                os.environ.pop(var, None)

            with ChildContext() as ctx:
                assert ctx.is_child is False
                assert ctx.parent_id is None
                assert ctx.child_id is None
                assert ctx.bridge is None

    def test_child_mode_properties(self):
        """Test ChildContext properties in child mode."""
        env = {
            ENV_PARENT_ID: "parent-test",
            ENV_CHILD_ID: "child-test",
            ENV_EXAMPLE_NAME: "demo_example",
        }
        with patch.dict(os.environ, env, clear=True):
            ctx = ChildContext()
            assert ctx.is_child is True
            assert ctx.parent_id == "parent-test"
            assert ctx.child_id == "child-test"
            assert ctx.example_name == "demo_example"

    def test_emit_to_parent_standalone(self):
        """Test emit_to_parent in standalone mode returns True."""
        with patch.dict(os.environ, {}, clear=True):
            os.environ.pop(ENV_PARENT_ID, None)

            ctx = ChildContext()
            result = ctx.emit_to_parent("test", {"data": "value"})
            assert result is True  # No-op success in standalone mode

    def test_on_parent_event_standalone(self):
        """Test on_parent_event in standalone mode returns no-op."""
        with patch.dict(os.environ, {}, clear=True):
            os.environ.pop(ENV_PARENT_ID, None)

            ctx = ChildContext()
            handler = MagicMock()
            unsubscribe = ctx.on_parent_event("test", handler)

            # Should return a callable
            assert callable(unsubscribe)
            unsubscribe()  # Should not raise

    def test_create_webview_standalone(self):
        """Test create_webview in standalone mode."""
        with patch.dict(os.environ, {}, clear=True):
            os.environ.pop(ENV_PARENT_ID, None)

            with patch("auroraview.child.WebView") as MockWebView:
                mock_webview = MagicMock()
                MockWebView.return_value = mock_webview

                ctx = ChildContext()
                ctx.create_webview(title="Test", width=800, height=600, html="<h1>Test</h1>")

                MockWebView.assert_called_once()
                call_kwargs = MockWebView.call_args[1]
                assert call_kwargs["title"] == "Test"
                assert call_kwargs["width"] == 800
                assert call_kwargs["height"] == 600

    def test_create_webview_child_mode_title_prefix(self):
        """Test create_webview adds title prefix in child mode."""
        env = {
            ENV_PARENT_ID: "parent-test",
            ENV_EXAMPLE_NAME: "MyDemo",
        }
        with patch.dict(os.environ, env, clear=True):
            with patch("auroraview.child.WebView") as MockWebView:
                mock_webview = MagicMock()
                MockWebView.return_value = mock_webview

                ctx = ChildContext()
                ctx.create_webview(title="Original Title")

                call_kwargs = MockWebView.call_args[1]
                assert call_kwargs["title"] == "MyDemo - Original Title"

    def test_handle_parent_command_close(self):
        """Test _handle_parent_command with close command."""
        ctx = ChildContext()
        mock_webview = MagicMock()
        ctx._webview = mock_webview

        ctx._handle_parent_command({"command": "close"})

        mock_webview.close.assert_called_once()

    def test_handle_parent_command_eval(self):
        """Test _handle_parent_command with eval command."""
        ctx = ChildContext()
        mock_webview = MagicMock()
        ctx._webview = mock_webview

        ctx._handle_parent_command({"command": "eval", "args": {"js": "console.log('test')"}})

        mock_webview.eval.assert_called_once_with("console.log('test')")

    def test_handle_parent_command_emit(self):
        """Test _handle_parent_command with emit command."""
        ctx = ChildContext()
        mock_webview = MagicMock()
        ctx._webview = mock_webview

        ctx._handle_parent_command(
            {"command": "emit", "args": {"event": "test_event", "data": {"key": "value"}}}
        )

        mock_webview.emit.assert_called_once_with("test_event", {"key": "value"})

    def test_handle_parent_command_no_webview(self):
        """Test _handle_parent_command when no webview."""
        ctx = ChildContext()
        ctx._webview = None

        # Should not raise
        ctx._handle_parent_command({"command": "close"})


class TestRunExample:
    """Tests for run_example function."""

    def test_run_example_standalone(self):
        """Test run_example in standalone mode."""
        with patch.dict(os.environ, {}, clear=True):
            os.environ.pop(ENV_PARENT_ID, None)

            mock_webview = MagicMock()
            create_func = MagicMock(return_value=mock_webview)
            on_ready = MagicMock()

            run_example(create_func, on_ready=on_ready)

            create_func.assert_called_once()
            on_ready.assert_called_once()
            mock_webview.show.assert_called_once()

    def test_run_example_child_mode(self):
        """Test run_example in child mode."""
        env = {
            ENV_PARENT_ID: "parent-run",
            ENV_CHILD_ID: "child-run",
        }
        with patch.dict(os.environ, env, clear=True):
            mock_webview = MagicMock()
            create_func = MagicMock(return_value=mock_webview)

            run_example(create_func)

            create_func.assert_called_once()
            # Verify context was passed
            ctx = create_func.call_args[0][0]
            assert ctx.is_child is True
            mock_webview.show.assert_called_once()


class TestEnvConstants:
    """Tests for environment variable constants."""

    def test_env_constants_defined(self):
        """Test that all env constants are defined."""
        assert ENV_PARENT_ID == "AURORAVIEW_PARENT_ID"
        assert ENV_PARENT_PORT == "AURORAVIEW_PARENT_PORT"
        assert ENV_CHILD_ID == "AURORAVIEW_CHILD_ID"
        assert ENV_EXAMPLE_NAME == "AURORAVIEW_EXAMPLE_NAME"
